// Ternary digit vector -*- c++ -*-

#include "TritVector.h"

#include <cassert>

#if CHAR_BIT == 8
static unsigned char trit_char[6] = { 1, 3, 9, 27, 81, 243 };
#endif // CHAR_BIT == 8

TritVector::TritVector (unsigned size, enum TritVector::Trit init) :
  mySize (size), myTrits (0)
{
  assert (mySize > 0);
  *(myTrits = new unsigned char[tritWords (mySize)]) = 0;
  unsigned i = TRITS;
  switch (init) {
  case False:
    while (i--) setFalse (i);
    break;
  case True:
    while (i--) setTrue (i);
    break;
  case Other:
    while (i--) setOther (i);
    break;
  default:
    assert (false);
  }
  memset (myTrits, *myTrits, tritWords (mySize));
}

enum TritVector::Trit
TritVector::operator[] (unsigned index) const
{
#if CHAR_BIT == 8
  return Trit (myTrits[index / TRITS] %
	       trit_char[index % TRITS + 1] /
	       trit_char[index % TRITS]);
#else // CHAR_BIT == 8
  return Trit ((myTrits[index / TRITS] >> (2 * (index % TRITS))) & 3);
#endif // CHAR_BIT == 8
}

void
TritVector::setTrue (unsigned index)
{
  unsigned char& word = myTrits[index / TRITS];
  unsigned offset = index % TRITS;
#if CHAR_BIT == 8
  assert (word < 243);
  word = word / trit_char[offset + 1] * trit_char[offset + 1] +
    word % trit_char[offset];
  word += trit_char[offset];
#else // CHAR_BIT == 8
  word &= ~3 << (2 * offset);
  word |= True << (2 * offset);
#endif // CHAR_BIT == 8
  assert ((*this)[index] == True);
}

void
TritVector::setFalse (unsigned index)
{
  unsigned char& word = myTrits[index / TRITS];
  unsigned offset = index % TRITS;
#if CHAR_BIT == 8
  assert (word < 243);
  word = word / trit_char[offset + 1] * trit_char[offset + 1] +
    word % trit_char[offset];
#else // CHAR_BIT == 8
  word &= ~3 << (2 * offset);
#endif // CHAR_BIT == 8
  assert ((*this)[index] == False);
}

void
TritVector::setOther (unsigned index)
{
  unsigned char& word = myTrits[index / TRITS];
  unsigned offset = index % TRITS;
#if CHAR_BIT == 8
  assert (word < 243);
  word = word / trit_char[offset + 1] * trit_char[offset + 1] +
    word % trit_char[offset];
  word += 2 * trit_char[offset];
#else // CHAR_BIT == 8
  word &= ~3 << (2 * offset);
  word |= Other << (2 * offset);
#endif // CHAR_BIT == 8
  assert ((*this)[index] == Other);
}
