// Buffer indexing expression (used only during parsing) -*- c++ -*-

#include "BufferIndex.h"

BufferIndex::BufferIndex (class Expression* buffer, class Expression* index) :
  myBuffer (buffer), myIndex (index)
{
  assert (myBuffer && myIndex);
  assert (myBuffer->getType ()->getKind () == Type::tBuffer);
  assert (myIndex->getType ()->getKind () == Type::tInt);
  assert (myBuffer->isBasic () && myIndex->isBasic ());

  setType (myBuffer->getType ());
}

BufferIndex::~BufferIndex ()
{
  myBuffer->destroy ();
  myIndex->destroy ();
}

#ifdef EXPR_CACHE
bool
BufferIndex::invalidate (const class VariableDefinition* variable) const
{
  bool invalidated = myBuffer->invalidate (variable);
  if (invalidated)
    Expression::invalidate ();
  return invalidated;
}

void
BufferIndex::do_clearValuation (const class Valuation* valuation) const
{
  myBuffer->clearValuation (valuation);
}
#endif // EXPR_CACHE

class Value*
BufferIndex::do_eval (const class Valuation* valuation) const
{
  return myBuffer->eval (valuation);
}

class Expression*
BufferIndex::ground (const class Valuation& valuation)
{
  return myBuffer->ground (valuation);
}

class Expression*
BufferIndex::substitute (class Substitution& substitution)
{
  return myBuffer->substitute (substitution);
}

#ifdef EXPR_CACHE
void
BufferIndex::associateVariables (const class Expression& expr,
				 bool assoc) const
{
  myBuffer->associateVariables (expr, assoc);
}
#endif // EXPR_CACHE

bool
BufferIndex::forVariables (bool (*operation)
			   (const class VariableDefinition&)) const
{
  return myBuffer->forVariables (operation);
}

#ifdef DEBUG
# include "util.h"

void
BufferIndex::prettyPrint (unsigned indent) const
{
  myBuffer->prettyPrint (indent);
  print (indent, "[\n");
  myIndex->prettyPrint (indent + 1);
  print (indent, "]\n");
}

#endif // DEBUG
