// Marking cardinality expression class -*- c++ -*-

#include "CardinalityExpression.h"
#include "PlaceMarking.h"
#include "Net.h"
#include "IntType.h"
#include "IntValue.h"
#include "Valuation.h"

CardinalityExpression::CardinalityExpression (class Expression* expr) :
  Expression (),
  myExpr (expr)
{
  assert (myExpr && !myExpr->isTemporal ());
  setType (Net::getIntType ());
}

CardinalityExpression::~CardinalityExpression ()
{
  myExpr->destroy ();
}

class Value*
CardinalityExpression::do_eval (const class Valuation* valuation) const
{
  if (class PlaceMarking* p = myExpr->meval (*valuation)) {
    int_t count = 0;
    for (PlaceMarking::const_iterator i = p->begin (); i != p->end (); i++) {
      card_t add = PlaceMarking::getCount (i);
      if (card_t (INT_T_MAX - count) < add) {
	valuation->flagOverflow ();
	delete p;
	return NULL;
      }
      else
	count += add;
    }
    delete p;
    return constrain (valuation, new class IntValue (getType (), count));
  }
  return NULL;
}

#ifdef DEBUG
# include "util.h"

void
CardinalityExpression::prettyPrint (unsigned indent) const
{
  print (indent, "cardinality (\n");
  myExpr->prettyPrint (indent + 1);
  print (indent, ")\n");
}

#endif // DEBUG
