// Marking cardinality expression class -*- c++ -*-

#ifndef CARDINALITYEXPRESSION_H_
# define CARDINALITYEXPRESSION_H_

# include "Expression.h"
# include "Marking.h"

/** Marking cardinality expression */
class CardinalityExpression : public Expression {
public:
  // Constructors and destructors

  /** Constructs a new CardinalityExpression
   * @param expr	the multiset-valued expression
   */
  CardinalityExpression (class Expression* expr);

private:
  /** Copy constructor */
  CardinalityExpression (const class CardinalityExpression& old);
  /** Assignment operator */
  class CardinalityExpression& operator=
  (const class CardinalityExpression& old);
protected:
  /** Destructs CardinalityExpression */
  ~CardinalityExpression ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eCardinality; }

  /** Determine whether this is a basic expression */
  bool isBasic () const { return false; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return false; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class CardinalityExpression& other) const {
    return *myExpr == *other.myExpr;
  }

  /** Ordering comparison operator */
  bool operator< (const class CardinalityExpression& other) const {
    return *myExpr < *other.myExpr;
  }

  /** Evaluate the expression
   * @param valuation	Variable substitutions
   * @return		Value of the expression, or NULL in case of error
   */
  class Value* do_eval (const class Valuation* valuation) const;

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** the multiset-valued expression */
  class Expression* myExpr;
};

#endif // CARDINALITYEXPRESSION_H_
