// constant class -*- c++ -*-

#ifndef CONSTANT_H_
# define CONSTANT_H_

# include "Expression.h"

/** Constant */
class Constant : public Expression {
public:
  // Constructors and destructors

  /** Constructs a new Constant
   * @param value	Value of the constant
   */
  Constant (class Value* value);

private:
  /** Copy constructor */
  Constant (const class Constant& old);
  /** Assignment operator */
  class Constant& operator= (const class Constant& old);
protected:
  /** Destructs Constant */
  ~Constant ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eConstant; }

  /** Determine whether this is a basic expression */
  bool isBasic () const { return true; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return false; }

# ifdef EXPR_CACHE
  /** Determine the constant value */
  const class Value* getValue () const { return Expression::myValue; }
# else // EXPR_CACHE
  /** Determine the constant value */
  const class Value* getValue () const { return myValue; }
# endif // EXPR_CACHE

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class Constant& other) const;

  /** Ordering comparison operator */
  bool operator< (const class Constant& other) const;

# ifndef EXPR_CACHE
  /** Evaluate the expression
   * @param valuation	Variable substitutions
   * @return		Value of the expression, or NULL in case of error
   */
  class Value* do_eval (const class Valuation* valuation) const;
# endif // EXPR_CACHE

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

  // Members
private:
#ifndef EXPR_CACHE
  /** Value of the constant */
  class Value* myValue;
#endif // EXPR_CACHE
};

#endif // CONSTANT_H_
