// Empty set expression -*- c++ -*-

#ifndef EMPTYSET_H_
# define EMPTYSET_H_

# include "Expression.h"

class EmptySet : public Expression {
public:
  // Constructors and destructors

  /** Constructor */
  EmptySet () {}
private:
  /** Copy constructor */
  EmptySet (const class EmptySet& old);
  /** Assignment operator */
  class EmptySet& operator= (const class EmptySet& old);
protected:
  /** Destructor */
  ~EmptySet () {}
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eEmptySet; }

  /**
   * Determine whether this is a basic expression containing
   * no temporal logic or set operations
   * @return	true if this is a basic expression
   */
  bool isBasic () const { return false; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return false; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class EmptySet& other) const {
    return true;
  }

  /** Ordering comparison operator */
  bool operator< (const class EmptySet& other) const {
    return false;
  }

  /** Evaluate the multiset expression
   * @param valuation	variable substitutions and the global marking
   * @return		the filtered marking
   */
  class PlaceMarking* meval (const class Valuation& valuation) const;

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG
};

#endif // EMPTYSET_H_
