// Function definition -*- c++ -*-

#include "Function.h"
#include "VariableDefinition.h"
#include "Expression.h"
#include "ExpressionList.h"
#include "Substitution.h"

Function::Function (char* name, unsigned arity,
		    class VariableDefinition** params,
		    class Expression* expr) :
  myName (name), myArity (arity), myParams (params), myExpr (expr)
{
  assert (name && expr);
}

Function::~Function ()
{
  delete[] myName;
  myExpr->destroy ();
  for (unsigned i = 0; i < myArity; i++)
    delete myParams[i];
  delete[] myParams;
}

class Expression*
Function::expand (class ExpressionList* params)
{
  if (myArity != (params ? params->size () : 0))
    return NULL;
  class Substitution substitution;
  for (unsigned i = 0; i < myArity; i++) {
    if (!(*params)[i]->isAssignable (myParams[i]->getType ()))
      return NULL;
    substitution.setExpr (*myParams[i], *(*params)[i]->copy ());
  }
  return myExpr->substitute (substitution);
}

#ifdef DEBUG
# include "util.h"

void
Function::prettyPrint (unsigned indent) const
{
  print (indent, "%s %s (\n",
	 myExpr->getType ()->getName ()
	 ? myExpr->getType ()->getName ()
	 : myExpr->getType ()->getSyntacticName (),
	 myName);
  for (unsigned i = 0; i < myArity; i++)
    print (indent + 1, "%s %s%s\n",
	   myParams[i]->getType ()->getName ()
	   ? myParams[i]->getType ()->getName ()
	   : myParams[i]->getType ()->getSyntacticName (),
	   myParams[i]->getName (),
	   i + 1 < myArity ? "," : ")");
  myExpr->prettyPrint (indent + 2);
}

#endif // DEBUG
