// Function definition -*- c++ -*-

#ifndef FUNCTION_H_
# define FUNCTION_H_

# include <cassert>

/** Function definition */
class Function {
public:
  // Constructors and destructors

  /** Constructor
   * @param name	Name of the function
   * @param arity	Arity (number of function parameters)
   * @param params	Parameter list of the function
   * @param expr	Function definition
   */
  Function (char* name, unsigned arity, class VariableDefinition** params,
	    class Expression* expr);
private:
  /** Copy constructor */
  Function (const class Function& old);
  /** Assignment operator */
  class Function& operator= (const class Function& old);
public:
  /** Destructor */
  ~Function ();

  // Accessors

  /** Determine the name of the function */
  const char* getName () const { return myName; }

  /** Determine the arity of the function */
  unsigned getArity () const { return myArity; }

  /** Get a formal parameter
   * @param index	Index to the parameter
   * @return		the formal parameter
   */
  const class VariableDefinition* operator[] (unsigned index) const {
    assert (index < myArity); return myParams[index];
  }

  // Other methods

  /** Expand the function
   * @param params	The expression list
   * @return		The expanded function, or NULL in case of error
   */
  class Expression* expand (class ExpressionList* params);

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** Name of the function */
  char* myName;
  /** Arity of the function */
  unsigned myArity;
  /** Function parameters */
  class VariableDefinition** myParams;
  /** Function definition */
  class Expression* myExpr;
};

#endif // FUNCTION_H_
