// Maria boolean negation operator class -*- c++ -*-

#include "NotExpression.h"
#include "Type.h"
#include "BoolValue.h"
#include "Constant.h"

NotExpression::NotExpression (class Expression* expr) :
  Expression (),
  myExpr (expr)
{
  setType (myExpr->getType ());
  assert (getType ()->getKind () == Type::tBool);
}

NotExpression::~NotExpression ()
{
  myExpr->destroy ();
}

#ifdef EXPR_CACHE
bool
NotExpression::invalidate (const class VariableDefinition* variable) const
{
  if (myExpr->invalidate (variable)) {
    Expression::invalidate ();
    return true;
  }

  return false;
}

void
NotExpression::do_clearValuation (const class Valuation* valuation) const
{
  myExpr->clearValuation (valuation);
}

class Value*
NotExpression::do_eval (const class Valuation* valuation) const
{
  const class Value* v = myExpr->const_eval (valuation);
  if (!v)
    return NULL;

  assert (v->getKind () == Value::vBool);
  return constrain (valuation, new class BoolValue
		    (getType (),
		     !static_cast<const class BoolValue*>(v)->getValue ()));
}
#else // EXPR_CACHE
class Value*
NotExpression::do_eval (const class Valuation* valuation) const
{
  if (class Value* v = myExpr->eval (valuation)) {
    assert (v->getKind () == Value::vBool);
    class BoolValue* bv = static_cast<class BoolValue*>(v);
    bv->setValue (!bv->getValue ());
    return constrain (valuation, bv);
  }
  else
    return NULL;
}
#endif // EXPR_CACHE

class Expression*
NotExpression::ground (const class Valuation& valuation)
{
  class Expression* e = myExpr->ground (valuation);
  if (!e)
    return NULL;
  if (e == myExpr) {
    e->destroy ();
    return copy ();
  }
  else {
    class Expression* expr = new class NotExpression (e);
    if (valuation.isOKExcept (Valuation::undefinedValue)) {
      if (class Value* v = expr->eval (&valuation)) {
	class Expression* c = new class Constant (v);
	expr->destroy ();
	return c;
      }
      else if (!valuation.isOKExcept (Valuation::undefinedValue)) {
	expr->destroy ();
	return NULL;
      }
    }

    return expr;
  }
}

class Expression*
NotExpression::substitute (class Substitution& substitution)
{
  class Expression* e = myExpr->substitute (substitution);
  if (e == myExpr) {
    e->destroy ();
    return copy ();
  }
  else
    return new class NotExpression (e);
}

#ifdef EXPR_CACHE
void
NotExpression::associateVariables (const class Expression& expr,
				   bool assoc) const
{
  myExpr->associateVariables (expr, assoc);
}
#endif // EXPR_CACHE

bool
NotExpression::forVariables (bool (*operation)
			     (const class VariableDefinition&)) const
{
  return myExpr->forVariables (operation);
}

#ifdef DEBUG
# include "util.h"

void
NotExpression::prettyPrint (unsigned indent) const
{
  print (indent, "!(\n");
  myExpr->prettyPrint (indent + 1);
  print (indent, ")\n");
}

#endif // DEBUG
