// Dynamic place contents -*- c++ -*-

#ifndef PLACECONTENTS_H_
# define PLACECONTENTS_H_

# include "Expression.h"

class PlaceContents : public Expression {
public:
  // Constructors and destructors

  /** Constructor
   * @param place	the place whose marking is to be determined
   */
  PlaceContents (const class Place* place);
private:
  /** Copy constructor */
  PlaceContents (const class PlaceContents& old);
  /** Assignment operator */
  class PlaceContents& operator= (const class PlaceContents& old);
protected:
  /** Destructor */
  ~PlaceContents ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return ePlaceContents; }

  /**
   * Determine whether this is a basic expression containing
   * no temporal logic or set operations
   * @return	true if this is a basic expression
   */
  bool isBasic () const { return false; }
  /**
   * Determine whether this is a temporal logic expression
   * @return	true if this is a temporal logic expression
   */
  bool isTemporal () const { return false; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class PlaceContents& other) const {
    return myIndex == other.myIndex;
  }

  /** Ordering comparison operator */
  bool operator< (const class PlaceContents& other) const {
    return myIndex < other.myIndex;
  }

  /** Evaluate the multiset expression
   * @param valuation	variable substitutions and the global marking
   * @return		the marking of myPlace
   */
  class PlaceMarking* meval (const class Valuation& valuation) const;

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The place index */
  const unsigned myIndex;
};

#endif // PLACECONTENTS_H_
