// Submarking expression -*- c++ -*-

#include "Submarking.h"
#include "VariableDefinition.h"
#include "Valuation.h"
#include "PlaceMarking.h"
#include "BoolValue.h"

Submarking::Submarking (class VariableDefinition* variable,
			class Expression* marking,
			class Expression* condition) :
  myVariable (variable), myMarking (marking), myCondition (condition)
{
  assert (myVariable && myCondition);
  assert (myMarking && myMarking->getType ());
  assert (!myMarking->isTemporal () && !myCondition->isTemporal ());
  setType (myMarking->getType ());
}

Submarking::~Submarking ()
{
  delete myVariable;
  myMarking->destroy ();
  myCondition->destroy ();
}

class PlaceMarking*
Submarking::meval (const class Valuation& valuation) const
{
  if (class PlaceMarking* p = myMarking->meval (valuation)) {
    class Valuation v;
    for (PlaceMarking::iterator i = p->begin (); i != p->end (); i++) {
      v.setValue (myVariable, PlaceMarking::getValue (i).copy ());
      if (class Value* val = myCondition->eval (&v)) {
	assert (val->getKind () == Value::vBool);
	bool pass = static_cast<class BoolValue*>(val)->getValue ();
	delete val;
	if (!pass)
	  p->remove (i);
      }
      else {
	delete p;
	return NULL;
      }
    }
    return p;
  }
  return NULL;
}

#ifdef DEBUG
# include "util.h"

void
Submarking::prettyPrint (unsigned indent) const
{
  print (indent, "submarking %s (\n", myVariable->getName ());
  myMarking->prettyPrint (indent + 1);
  print (indent, ")\n");
  myCondition->prettyPrint (indent + 1);
}

#endif // DEBUG
