// Submarking expression -*- c++ -*-

#ifndef SUBMARKING_H_
# define SUBMARKING_H_

# include "Expression.h"

class Submarking : public Expression {
public:
  // Constructors and destructors

  /** Constructor
   * @param variable	a variable that will run through marking
   * @param marking	a multi-set expression
   * @param condition	the criterion for filtering the multiset items
   */
  Submarking (class VariableDefinition* variable,
	      class Expression* marking, class Expression* condition);
private:
  /** Copy constructor */
  Submarking (const class Submarking& old);
  /** Assignment operator */
  class Submarking& operator= (const class Submarking& old);
protected:
  /** Destructor */
  ~Submarking ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eSubmarking; }

  /**
   * Determine whether this is a basic expression containing
   * no temporal logic or set operations
   * @return	true if this is a basic expression
   */
  bool isBasic () const { return false; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return false; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class Submarking& other) const {
    return myVariable == other.myVariable &&
      *myMarking == *other.myMarking && *myCondition == *other.myCondition;
  }

  /** Ordering comparison operator */
  bool operator< (const class Submarking& other) const {
    if (myVariable < other.myVariable) return true;
    if (other.myVariable < myVariable) return false;
    if (*myMarking < *other.myMarking) return true;
    if (*other.myMarking < *myMarking) return false;
    return *myCondition < *other.myCondition;
  }

  /** Evaluate the multiset expression
   * @param valuation	variable substitutions and the global marking
   * @return		the filtered marking
   */
  class PlaceMarking* meval (const class Valuation& valuation) const;

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The iterator variable */
  class VariableDefinition* myVariable;
  /** The marking expression */
  class Expression* myMarking;
  /** The submarking criterion */
  class Expression* myCondition;
};

#endif // SUBMARKING_H_
