// Substitution (expression distribution for variables) -*- c++ -*-

#include "Substitution.h"
#include "Expression.h"
#include "VariableDefinition.h"

Substitution::Substitution () :
  myExprs ()
{
}

Substitution::~Substitution ()
{
  for (iterator i = myExprs.begin (); i != myExprs.end (); i++)
    i->second->destroy ();
}

class Expression*
Substitution::getExpr (const class VariableDefinition& var)
{
  const_iterator i = myExprs.find (&var);
  return i == myExprs.end () ? NULL : i->second->copy ();
}

void
Substitution::setExpr (const class VariableDefinition& var,
		       class Expression& expr)
{
  assert (expr.isAssignable (var.getType ()));
  assert (!getExpr (var));
  myExprs[&var] = &expr;
}

#ifdef DEBUG
# include "util.h"

void
Substitution::prettyPrint (unsigned indent) const
{
  print (indent, "Substitution {\n");
  for (const_iterator i = myExprs.begin (); i != myExprs.end (); i++) {
    print (indent, "%s:\n", i->first->getName ());
    i->second->prettyPrint (indent + 1);
  }
  print (indent, "}\n");
}

#endif // DEBUG
