// Substitution (expression distribution for variables) -*- c++ -*-

#ifndef SUBSTITUTION_H_
# define SUBSTITUTION_H_

# include <map>

/** Substitution (expression distribution for variables) */
class Substitution {
  /** Pointer comparator */
  struct vptr {
    bool operator () (const class VariableDefinition* v1,
		      const class VariableDefinition* v2) const {
      return v1 < v2;
    }
  };
  typedef std::map<const class VariableDefinition*,class Expression*,vptr> Map;
  typedef Map::iterator iterator;
  typedef Map::const_iterator const_iterator;

public:
  // Constructors and destructors

  /** Constructs Substitution */
  Substitution ();
private:
  /** Copy constructor */
  Substitution (const class Substitution& old);
  /** Assignment operator */
  class Substitution& operator= (const class Substitution& old);
public:
  /** Destructs Substitution */
  ~Substitution ();

  // Accessors

  /** Get the expression of a variable
   * @param var		identifies the variable
   * @return		the expression, or NULL if there is none
   */
  const class Expression* getExpr (const class VariableDefinition& var) const {
    const_iterator i = myExprs.find (&var);
    return i == myExprs.end () ? NULL : i->second;
  }

  /** Get the expression of a variable
   * @param var		identifies the variable
   * @return		the expression, or NULL if there is none
   */
  class Expression* getExpr (const class VariableDefinition& var);

  /** Assign an expression to a variable.
   * @param var		identifies the variable
   * @param expr	expression to be assigned
   */
  void setExpr (const class VariableDefinition& var, class Expression& expr);

  // Other methods

# ifdef DEBUG
  /** Print this object
   * @param indent	level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

  // Members
private:
  /** The expressions sorted by variables */
  Map myExprs;
};

#endif // SUBSTITUTION_H_
