// Maria temporal binary operator class -*- c++ -*-

#include "TemporalBinop.h"
#include "TransitionQualifier.h"
#include "BoolType.h"
#include "Net.h"

TemporalBinop::TemporalBinop (enum Op op,
			      class Expression* qual,
			      class Expression* left,
			      class Expression* right) :
  Expression (),
  myOp (op), myQual (qual), myLeft (left), myRight (right)
{
  assert (myLeft->getType ()->getKind () == Type::tBool);
  assert (myRight->getType ()->getKind () == Type::tBool);
  setType (Net::getBoolType ());
}

TemporalBinop::~TemporalBinop ()
{
  myQual->destroy ();
  myLeft->destroy ();
  myRight->destroy ();
}

const char*
TemporalBinop::getOpString () const
{
  switch (getOp ()) {
  case Until:
    return "until";
  case Release:
    return "release";
  }

  return "???";
}

#ifdef DEBUG
# include "util.h"

void
TemporalBinop::prettyPrint (unsigned indent) const
{
  print (indent, "(\n");
  myLeft->prettyPrint (indent + 1);
  print (indent, ")\n");
  if (myQual) {
    print (indent, "%s(\n", getOpString ());
    myQual->prettyPrint (indent + 1);
    print (indent, ")\n");
  }
  else
    print (indent, "%s\n", getOpString ());
  print (indent, "(\n");
  myRight->prettyPrint (indent + 1);
  print (indent, ")\n");
}

#endif // DEBUG
