// Maria temporal quantifier class -*- c++ -*-

#ifndef TEMPORALQUANTIFIER_H_
# define TEMPORALQUANTIFIER_H_

# include "Expression.h"

class TemporalQuantifier : public Expression {
public:
  /** Temporal quantifiers */
  enum Quantifier { Always, Exists };

  // Constructors and destructors

  /** Constructs a new TemporalQuantifier
   * @param quantifier	path quantifier
   * @param expr	Subexpression of the operator
   */
  TemporalQuantifier (enum Quantifier quantifier, class Expression* expr);

private:
  /** Copy constructor */
  TemporalQuantifier (const class TemporalQuantifier& old);
  /** Assignment operator */
  class TemporalQuantifier& operator= (const class TemporalQuantifier& old);
protected:
  /** Destructs TemporalQuantifier */
  ~TemporalQuantifier ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eTemporalQuantifier; }

  /** Determine whether this is a basic expression */
  bool isBasic () const { return false; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return true; }

  /** Determine the path quantifier */
  enum Quantifier getQuantifier () const { return myQuantifier; }
  /** Determine the string representation of the quantifier */
  const char* getQuantifierString () const;
  /** Determine the subexpression */
  const class Expression* getExpr () const { return myExpr; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class TemporalQuantifier& other) const {
    return myQuantifier == other.myQuantifier && *myExpr == *other.myExpr;
  }

  /** Ordering comparison operator */
  bool operator< (const class TemporalQuantifier& other) const {
    if (myQuantifier < other.myQuantifier) return true;
    if (other.myQuantifier < myQuantifier) return false;
    return *myExpr < *other.myExpr;
  }

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The operator */
  enum Quantifier myQuantifier;
  /** The subexpression */
  class Expression* myExpr;
};

#endif // TEMPORALQUANTIFIER_H_
