// Maria temporal unary operator class -*- c++ -*-

#include "TemporalUnop.h"
#include "TransitionQualifier.h"
#include "BoolType.h"
#include "Net.h"

TemporalUnop::TemporalUnop (enum Op op,
			    class Expression* qual,
			    class Expression* expr) :
  Expression (),
  myOp (op), myQual (qual), myExpr (expr)
{
  assert (myExpr->getType ()->getKind () == Type::tBool);
  setType (Net::getBoolType ());
}

TemporalUnop::~TemporalUnop ()
{
  myQual->destroy ();
  myExpr->destroy ();
}

const char*
TemporalUnop::getOpString () const
{
  switch (getOp ()) {
  case Finally:
    return "<>";
  case Globally:
    return "[]";
  case Next:
    return "()";
  }

  return "???";
}

#ifdef DEBUG
# include "util.h"

void
TemporalUnop::prettyPrint (unsigned indent) const
{
  if (myQual) {
    print (indent, "%s(\n", getOpString ());
    myQual->prettyPrint (indent + 1);
    print (indent, ")\n");
  }
  else
    print (indent, "%s\n", getOpString ());

  myExpr->prettyPrint (indent);
}

#endif // DEBUG
