// Maria temporal unary operator class -*- c++ -*-

#ifndef TEMPORALUNOP_H_
# define TEMPORALUNOP_H_

# include "Expression.h"

class TemporalUnop : public Expression {
public:
  /** Temporal unary operators */
  enum Op { Finally, Globally, Next };

  // Constructors and destructors

  /** Constructs a new TemporalUnop
   * @param op		Operator
   * @param qual	Transition instance qualifier expression
   * @param expr	Subexpression of the operator
   */
  TemporalUnop (enum Op op,
		class Expression* qual,
		class Expression* expr);

private:
  /** Copy constructor */
  TemporalUnop (const class TemporalUnop& old);
  /** Assignment operator */
  class TemporalUnop& operator= (const class TemporalUnop& old);
protected:
  /** Destructs TemporalUnop */
  ~TemporalUnop ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eTemporalUnop; }

  /** Determine whether this is a basic expression */
  bool isBasic () const { return false; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return true; }

  /** Determine the operator */
  enum Op getOp () const { return myOp; }
  /** Determine the string representation of the operator */
  const char* getOpString () const;
  /** Determine the transition instance qualifier expression */
  const class Expression* getQual () const { return myQual; }
  /** Determine the subexpression */
  const class Expression* getExpr () const { return myExpr; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class TemporalUnop& other) const {
    return myOp == other.myOp && *myQual == *other.myQual &&
      *myExpr == *other.myExpr;
  }

  /** Ordering comparison operator */
  bool operator< (const class TemporalUnop& other) const {
    if (myOp < other.myOp) return true;
    if (other.myOp < myOp) return false;
    if (*myQual < *other.myQual) return true;
    if (*other.myQual < *myQual) return false;
    return *myExpr < *other.myExpr;
  }

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The operator */
  enum Op myOp;
  /** Transition instance qualifier */
  class Expression* myQual;
  /** The subexpression */
  class Expression* myExpr;
};

#endif // TEMPORALUNOP_H_
