// Maria transition qualifier expression class -*- c++ -*-

#ifndef TRANSITIONQUALIFIER_H_
# define TRANSITIONQUALIFIER_H_

# include "Expression.h"

/** Transition instance qualifier expression */
class TransitionQualifier : public Expression {
public:
  // Constructors and destructors

  /** Constructs a new TransitionQualifier
   * @param transition	The transition whose instances are being qualified
   * @param expr	Qualifying expression for the transition instances
   */
  TransitionQualifier (const class Transition* transition,
		       class Expression* expr);
private:
  /** Copy constructor */
  TransitionQualifier (const class TransitionQualifier& old);
  /** Assignment operator */
  class TransitionQualifier& operator= (const class TransitionQualifier& old);
protected:
  /** Destructs TransitionQualifier */
  ~TransitionQualifier ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eTransitionQualifier; }
  /** Determine the transition */
  const class Transition* getTransition () const { return myTransition; }
  /** Determine the qualifier expression */
  const class Expression* getExpr () const { return myExpr; }

  /** Determine whether this is a basic expression */
  bool isBasic () const { return false; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return true; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class TransitionQualifier& other) const {
    return myTransition == other.myTransition && *myExpr == *other.myExpr;
  }

  /** Ordering comparison operator */
  bool operator< (const class TransitionQualifier& other) const {
    if (myTransition < other.myTransition)
      return true;
    if (other.myTransition < myTransition)
      return false;
    return *myExpr < *other.myExpr;
  }

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

  // TO DO: bool match (class Valuation*) or similar method

private:
  /** The transition */
  const class Transition* myTransition;
  /** The qualifier expression */
  class Expression* myExpr;
};

#endif // TRANSITIONQUALIFIER_H_
