// undefined expression class -*- c++ -*-

#ifndef UNDEFINED_H_
# define UNDEFINED_H_

# include "Expression.h"

/** Undefined expression */
class Undefined : public Expression {
public:
  /** Fatalness levels */
  enum Fatalness {
    /** issue an error message; do not fire the transition instance */
    fError,
    /** issue a fatal error message; stop the graph generation */
    fFatalError
  };

  // Constructors and destructors

  /** Constructs a new Undefined
   * @param fatalness	Fatalness level
   */
  Undefined (enum Fatalness fatalness);

private:
  /** Copy constructor */
  Undefined (const class Undefined& old);
  /** Assignment operator */
  class Undefined& operator= (const class Undefined& old);
protected:
  /** Destructs Undefined */
  ~Undefined ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eUndefined; }

  /** Determine whether this is a basic expression */
  bool isBasic () const { return true; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return false; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class Undefined& other) const {
    return myFatalness == other.myFatalness;
  }

  /** Ordering comparison operator */
  bool operator< (const class Undefined& other) const {
    return myFatalness < other.myFatalness;
  }

  /** Evaluate the expression
   * @param valuation	Variable substitutions
   * @return		Value of the expression, or NULL in case of error
   */
  class Value* do_eval (const class Valuation* valuation) const;

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** Fatalness level */
  enum Fatalness myFatalness;
};

#endif // UNDEFINED_H_
