// Maria union expression class -*- c++ -*-

#include "UnionExpression.h"
#include "UnionType.h"
#include "UnionValue.h"
#include "Constant.h"

UnionExpression::UnionExpression (const class Type* type,
				  class Expression* expr, card_t index) :
  Expression (),
  myUnionType (type), myExpr (expr), myIndex (index)
{
  assert (type && type->getKind () == Type::tUnion);
  assert (myExpr->getType ()->isAssignable
	  ((*static_cast<const class UnionType*>(type))[index]));
  assert (myExpr->isBasic ());
  Expression::setType (type);
}

UnionExpression::~UnionExpression ()
{
  myExpr->destroy ();
}

#ifdef EXPR_CACHE
bool
UnionExpression::invalidate (const class VariableDefinition* variable) const
{
  if (myExpr->invalidate (variable)) {
    Expression::invalidate ();
    return true;
  }

  return false;
}

void
UnionExpression::do_clearValuation (const class Valuation* valuation) const
{
  myExpr->clearValuation (valuation);
}
#endif // EXPR_CACHE

class Value*
UnionExpression::do_eval (const class Valuation* valuation) const
{
  class Value* v = myExpr->eval (valuation);
  if (!v)
    return NULL;

  return new class UnionValue (myUnionType, myIndex, v);
}

class Expression*
UnionExpression::ground (const class Valuation& valuation)
{
  class Expression* e = myExpr->ground (valuation);
  if (!e)
    return NULL;
  if (e == myExpr) {
    e->destroy ();
    return copy ();
  }

  if (valuation.isOKExcept (Valuation::undefinedValue)) {
    if (class Value* v = e->eval (&valuation)) {
      e->destroy ();
      return new class Constant (new class UnionValue
				 (myUnionType, myIndex, v));
    }
    else if (!valuation.isOKExcept (Valuation::undefinedValue)) {
      e->destroy ();
      return NULL;
    }
  }

  return new class UnionExpression (myUnionType, e, myIndex);
}

class Expression*
UnionExpression::substitute (class Substitution& substitution)
{
  class Expression* e = myExpr->substitute (substitution);
  if (e == myExpr) {
    e->destroy ();
    return copy ();
  }
  else
    return new class UnionExpression (myUnionType, e, myIndex);
}

#ifdef EXPR_CACHE
void
UnionExpression::associateVariables (const class Expression& expr,
				     bool assoc) const
{
  myExpr->associateVariables (expr, assoc);
}
#endif // EXPR_CACHE

bool
UnionExpression::forVariables (bool (*operation)
			       (const class VariableDefinition&)) const
{
  return myExpr->forVariables (operation);
}

const class VariableDefinition*
UnionExpression::getLvalue (const class Value*& value,
			    class Valuation& valuation) const
{
  return myExpr->getLvalue (value, valuation);
}

bool
UnionExpression::isLvalue (const class VariableDefinition* variable) const
{
  return myExpr->isLvalue (variable);
}

#ifdef DEBUG
# include "util.h"

void
UnionExpression::prettyPrint (unsigned indent) const
{
  print (indent, "%s = \n",
	 static_cast<const class UnionType*>(myUnionType)
	 ->getComponentName (myIndex));
  myExpr->prettyPrint (indent + 1);
}

#endif // DEBUG
