// Maria union type expression class -*- c++ -*-

#include "UnionTypeExpression.h"
#include "UnionType.h"
#include "UnionValue.h"
#include "BoolType.h"
#include "BoolValue.h"
#include "Net.h"
#include "Constant.h"

UnionTypeExpression::UnionTypeExpression (class Expression* expr,
					  card_t index) :
  myExpr (expr), myIndex (index)
{
  assert (myExpr && myExpr->getType ());
  assert (myExpr->getType ()->getKind () == Type::tUnion);
  assert ((*static_cast<const class UnionType*>(myExpr->getType ()))[index] !=
	  NULL);
  assert (myExpr->isBasic ());

  setType (Net::getBoolType ());
}

UnionTypeExpression::~UnionTypeExpression ()
{
  myExpr->destroy ();
}

#ifdef EXPR_CACHE
bool
UnionTypeExpression::invalidate (const class VariableDefinition* variable)
const
{
  if (myExpr->invalidate (variable)) {
    Expression::invalidate ();
    return true;
  }

  return false;
}

void
UnionTypeExpression::do_clearValuation (const class Valuation* valuation) const
{
  myExpr->clearValuation (valuation);
}

class Value*
UnionTypeExpression::do_eval (const class Valuation* valuation) const
{
  const class Value* v = myExpr->const_eval (valuation);
  if (!v)
    return NULL;

  assert (v->getKind () == Value::vUnion);
  bool is = static_cast<const class UnionValue*>(v)->getIndex () == myIndex;

  return constrain (valuation, new class BoolValue (getType (), is));
}
#else // EXPR_CACHE
class Value*
UnionTypeExpression::do_eval (const class Valuation* valuation) const
{
  class Value* v = myExpr->eval (valuation);
  if (!v)
    return NULL;

  assert (v->getKind () == Value::vUnion);
  bool is = static_cast<class UnionValue*>(v)->getIndex () == myIndex;
  delete v;

  return constrain (valuation, new class BoolValue (getType (), is));
}
#endif // EXPR_CACHE

class Expression*
UnionTypeExpression::ground (const class Valuation& valuation)
{
  class Expression* e = myExpr->ground (valuation);
  if (!e)
    return NULL;
  if (e == myExpr) {
    e->destroy ();
    return copy ();
  }
  else {
    class Expression* expr = new class UnionTypeExpression (e, myIndex);
    if (valuation.isOKExcept (Valuation::undefinedValue)) {
      if (class Value* v = expr->eval (&valuation)) {
	class Expression* c = new class Constant (v);
	expr->destroy ();
	return c;
      }
      else if (!valuation.isOKExcept (Valuation::undefinedValue)) {
	expr->destroy ();
	return NULL;
      }
    }

    return expr;
  }
}

class Expression*
UnionTypeExpression::substitute (class Substitution& substitution)
{
  class Expression* e = myExpr->substitute (substitution);
  if (e == myExpr) {
    e->destroy ();
    return copy ();
  }
  else
    return new class UnionTypeExpression (e, myIndex);
}

#ifdef EXPR_CACHE
void
UnionTypeExpression::associateVariables (const class Expression& expr,
					 bool assoc) const
{
  myExpr->associateVariables (expr, assoc);
}
#endif // EXPR_CACHE

bool
UnionTypeExpression::forVariables (bool (*operation)
				   (const class VariableDefinition&)) const
{
  return myExpr->forVariables (operation);
}

#ifdef DEBUG
# include "util.h"

void
UnionTypeExpression::prettyPrint (unsigned indent) const
{
  myExpr->prettyPrint (indent);
  print (indent, "is %s\n",
	 static_cast<const class UnionType*>(myExpr->getType ())
	 ->getComponentName (myIndex));
}

#endif // DEBUG
