// Test for the presence of all vector components -*- c++ -*-

#include "VectorDefined.h"
#include "VectorType.h"
#include "VectorValue.h"
#include "Valuation.h"
#include "Net.h"
#include "BoolType.h"
#include "BoolValue.h"
#include "Constant.h"

VectorDefined::VectorDefined (class Expression* vector) :
  Expression (),
  myVector (vector)
{
  assert (myVector && myVector->getType ());
  assert (myVector->getType ()->getKind () == Type::tVector &&
	  static_cast<const class VectorType*>(myVector->getType ())
	  ->isOptional ());
  assert (myVector->isBasic ());
  setType (Net::getBoolType ());
}

VectorDefined::~VectorDefined ()
{
  myVector->destroy ();
}

#ifdef EXPR_CACHE
bool
VectorDefined::invalidate (const class VariableDefinition* variable) const
{
  const bool invalidated = myVector->invalidate (variable);
  if (invalidated)
    Expression::invalidate ();
  return invalidated;
}

void
VectorDefined::do_clearValuation (const class Valuation* valuation) const
{
  myVector->clearValuation (valuation);
}

class Value*
VectorDefined::do_eval (const class Valuation* valuation) const
{
  const class Value* const vector = myVector->const_eval (valuation);
  if (!vector)
    return NULL;
  assert (vector->getType () == myVector->getType ());

  for (card_t i = static_cast<const class VectorType*>(myVector->getType ())
	 ->getSize (); i--; ) {
    if (!(*static_cast<const class VectorValue*>(vector))[i])
      return constrain (valuation, new class BoolValue (getType (), false));
  }

  return constrain (valuation, new class BoolValue (getType (), true));
}
#else
class Value*
VectorDefined::do_eval (const class Valuation* valuation) const
{
  class Value* vector = myVector->eval (valuation);
  if (!vector)
    return NULL;
  assert (vector->getType () == myVector->getType ());

  for (card_t i = static_cast<const class VectorType*>(myVector->getType ())
	 ->getSize (); i--; ) {
    if (!(*static_cast<const class VectorValue*>(vector))[i]) {
      delete vector;
      return constrain (valuation, new class BoolValue (getType (), false));
    }
  }

  delete vector;
  return constrain (valuation, new class BoolValue (getType (), true));
}
#endif // EXPR_CACHE

class Expression*
VectorDefined::ground (const class Valuation& valuation)
{
  class Expression* vector = myVector->ground (valuation);

  if (!vector) {
    vector->destroy ();
    return NULL;
  }

  if (vector == myVector) {
    vector->destroy ();
    return copy ();
  }
  else {
    class Expression* expr = new class VectorDefined (vector);
    if (valuation.isOKExcept (Valuation::undefinedValue)) {
      if (class Value* v = expr->eval (&valuation)) {
	class Expression* c = new class Constant (v);
	expr->destroy ();
	return c;
      }
      else if (!valuation.isOKExcept (Valuation::undefinedValue)) {
	expr->destroy ();
	return NULL;
      }
    }

    return expr;
  }
}

class Expression*
VectorDefined::substitute (class Substitution& substitution)
{
  class Expression* vector = myVector->substitute (substitution);

  if (vector == myVector) {
    vector->destroy ();
    return copy ();
  }
  else
    return new class VectorDefined (vector);
}

#ifdef EXPR_CACHE
void
VectorDefined::associateVariables (const class Expression& expr,
				   bool assoc) const
{
  myVector->associateVariables (expr, assoc);
}
#endif // EXPR_CACHE

bool
VectorDefined::forVariables (bool (*operation)
			     (const class VariableDefinition&)) const
{
  return myVector->forVariables (operation);
}

#ifdef DEBUG
# include "util.h"

void
VectorDefined::prettyPrint (unsigned indent) const
{
  print (indent, "?\n");
  myVector->prettyPrint (indent);
}

#endif // DEBUG
