// Test for the presence of all vector components -*- c++ -*-

#ifndef VECTORDEFINED_H_
# define VECTORDEFINED_H_

# include "Expression.h"

/** Test for the presence of all vector components */
class VectorDefined : public Expression {
public:
  // Constructors and destructors

  /** Constructor
   * @param vector	vector to be queried
   */
  VectorDefined (class Expression* vector);
private:
  /** Copy constructor */
  VectorDefined (const class VectorDefined& old);
  /** Assignment operator */
  class VectorDefined& operator= (const class VectorDefined& old);
protected:
  /** Destructor */
  ~VectorDefined ();
public:

  // Accessors

  /** Determine the type of the expression */
  enum Expression::Kind getKind () const { return eVectorDefined; }

  /** Determine whether this is a basic expression */
  bool isBasic () const { return true; }
  /** Determine whether this is a temporal logic expression */
  bool isTemporal () const { return false; }

  // Other methods

  /** Equality comparison operator */
  bool operator== (const class VectorDefined& other) const {
    return *myVector == *other.myVector;
  }

  /** Ordering comparison operator */
  bool operator< (const class VectorDefined& other) const {
    return *myVector < *other.myVector;
  }

  /** Conditionally invalidate the cached value of the expression,
   * provided that it depends on the value of a variable
   * @param variable	variable the value should depend on
   * @return		true if the expression was invalidated
   */
  bool invalidate (const class VariableDefinition* variable) const;

  /** Recursively clear the valuation used for computing the cached value,
   * provided that it equals with the specified one
   * @param valuation	the valuation to be cleared
   */
  void do_clearValuation (const class Valuation* valuation) const;

  /** Evaluate the expression
   * @param valuation	Variable substitutions
   * @return		Value of the expression, or NULL in case of error
   */
  class Value* do_eval (const class Valuation* valuation) const;

  /** Partially evaluate the expression using a valuation
   * @param valuation	Variable substitutions
   * @return		grounded expression, or NULL in case of error
   */
  class Expression* ground (const class Valuation& valuation);

  /** Substitute some variables in the expression with expressions
   * @param substitution	Variable substitutions
   * @return			substituted expression
   */
  class Expression* substitute (class Substitution& substitution);

  /** Associate the variables in the (sub)expression with the given expression
   * @param expr	expression the variables are to be associated with
   * @param assoc	false=disassociate
   */
  void associateVariables (const class Expression& expr, bool assoc) const;

  /** Perform an operation on all variables of the expression
   * @param operation	operation to be performed (return false on failure)
   * @return		true if all operations succeeded
   */
  bool forVariables (bool (*operation)
		     (const class VariableDefinition&)) const;

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

  // Members

private:
  /** The vector expression */
  class Expression* myVector;
};

#endif // VECTORDEFINED_H_
