// Event in a reachability graph -*- c++ -*-

#include "Event.h"
#include "State.h"

#include "Net.h"
#include "Transition.h"
#include "Valuation.h"
#include "VariableDefinition.h"

#include "BitBuffer.h"

void
Event::encode (const class Net& net,
	       const class Transition& transition,
	       const class Valuation& valuation,
	       class BitPacker& buf)
{
  assert (buf.empty ());
  assert (net.getTransition (transition.getIndex ()) == &transition);
  assert (valuation.isOK ());
  if (net.getNumTransitions () > 1)
    buf.append (transition.getIndex (), log2 (net.getNumTransitions ()));
  else
    assert (net.getNumTransitions () == 1);

  for (Transition::const_iterator i = transition.begin ();
       i != transition.end (); i++) {
    const class Value* value = valuation.getValue (i->second);
    buf.append (!value, 1);
    if (value)
      buf.append (*value);
  }
}

class Valuation*
Event::decode (const class Net& net, const word_t* data,
	       const class Transition*& transition)
{
  class BitUnpacker buf (data);

  transition = net.getTransition
    (net.getNumTransitions () > 1
     ? buf.extract (log2 (net.getNumTransitions ()))
     : 0);
  class Valuation* valuation = new class Valuation;

  for (Transition::const_iterator i = transition->begin ();
       i != transition->end (); i++)
    if (!buf.extract (1))
      valuation->setValue (i->second, buf.extract (*i->second->getType ()));

  return valuation;
}
