// Event in a reachability graph -*- c++ -*-

#ifndef EVENT_H_
# define EVENT_H_

# include "BitBuffer.h"

/** Event in a reachability graph */
class Event {
public:
  /** Constructor
   * @param successor	the state this event is leading to
   * @param offset	offset of the event to the disk file
   */
  Event (const class State& successor,
	 long offset) : mySuccessor (successor), myOffset (offset) {}
private:
  /** Copy constructor */
  Event (const class Event& old);
  /** Assignment operator */
  class Event& operator= (const class Event& old);
public:
  /** Destructor */
  ~Event () {}

  /** Determine the successor state */
  const class State& getSuccessor () const { return mySuccessor; }
  /** Determine the offset to the disk file */
  long getOffset () const { return myOffset; }

  /** Encode an event
   * @param net		the net whose event this is
   * @param transition	the transition
   * @param valuation	the valuation of the transition
   * @param buf		buffer to receive the encoded event
   */
  static void encode (const class Net& net,
		      const class Transition& transition,
		      const class Valuation& valuation,
		      class BitPacker& buf);

  /** Decode an event
   * @param net		the net whose event this is
   * @param data	the encoded data
   * @param transition	(output parameter) the transition
   * @return		the variable bindings for the transition
   */
  static class Valuation* decode (const class Net& net,
				  const word_t* data,
				  const class Transition*& transition);

  // Members
private:
  /** Successor state */
  const class State& mySuccessor;
  /** Offset to the arc file */
  const long myOffset;
};

#endif // EVENT_H_
