// State of reachability analysis -*- c++ -*-

#ifndef SEARCH_H_
# define SEARCH_H_

# include "State.h"
# include <list>

/** State of reachability analysis */
class Search {
public:
  /** Default constructor */
  Search (): myState (0), myHash (0) {}
  /** Constructor
   * @param state	state
   * @param h		hash value of the state
   */
  Search (const class State& state, size_t h) :
    myState (&state), myHash (h) {}
  /** Copy constructor */
  Search (const class Search& old) :
    myState (old.myState), myHash (old.myHash) {}
  /** Assignment operator */
  class Search& operator= (const class Search& old) {
    myState = old.myState, myHash = old.myHash;
    return *this;
  }
public:
  /** Destructor */
  ~Search () {}

  /** Determine whether a state is set */
  bool isState () const { return myState && true; }
  /** Clear the state */
  void clearState () { myState = 0; }
  /** Determine the state */
  const class State& getState () const { return *myState; }
  /** Determine the number of the state */
  card_t getNumber () const { return myState->getNumber (); }
  /** Determine the hash value of the state */
  size_t getHash () const { return myHash; }

  /** Equality comparison */
  bool operator== (const class Search& other) const {
    return myState->getNumber () == other.myState->getNumber ();
  }
  /** Less-than comparison */
  bool operator< (const class Search& other) const {
    return myState->getNumber () < other.myState->getNumber ();
  }

private:
  /** The state */
  const class State* myState;
  /** The hash value */
  size_t myHash;
};

/** List of search states */
class SearchList {
public:
  typedef std::list<class Search> List;
  typedef List::iterator iterator;
  typedef List::const_iterator const_iterator;

  /** Constructor */
  SearchList () : myList () {}
  /** Copy constructor */
  SearchList (const class SearchList& old) : myList (old.myList) {}
  /** Assignment operator */
  class SearchList& operator= (const class SearchList& old) {
    myList = old.myList;
    return *this;
  }
  /** Destructor */
  ~SearchList () {}

  /** Determine whether the list is empty */
  bool empty () const { return myList.empty (); }
  /** Determine the size of the list */
  size_t size () const { return myList.size (); }

  /** Clear the list */
  void clear () { myList.clear (); }
  /** Insert an item to the list */
  void push (class Search item) { myList.push_front (item); }
  /** Fetch an item
   * @param tail	flag: retrieve from tail of list instead of head */
  class Search pop (bool tail = false) {
    iterator i;
    if (tail)
      (i = myList.end ())--;
    else
      i = myList.begin ();
    class Search search = *i;
    myList.erase (i);
    return search;
  }

private:
  /** The search list */
  List myList;
};

#endif // SEARCH_H_
