// State in a reachability graph -*- c++ -*-

#include "State.h"
#include "Event.h"

#include "Net.h"
#include "Place.h"

#include "GlobalMarking.h"
#include "PlaceMarking.h"

#include "StructType.h"
#include "StructValue.h"
#include "UnionType.h"
#include "UnionValue.h"
#include "VectorType.h"
#include "VectorValue.h"

void
State::connect (class Event& event)
{
  if (getNumEvents ()) {
    class Event** events = new class Event*[getNumEvents () + 1];
    memcpy (events, myEvents, getNumEvents () * sizeof *events);
    delete[] myEvents;
    myEvents = events;
  }
  else {
    assert (!myEvents);
    myEvents = new class Event*[1];
  }
  myEvents[getNumEvents ()] = &event;
  myNumEvents++;
}

void
State::encode (const class GlobalMarking& m, class BitPacker& buf)
{
  assert (buf.empty ());
  for (unsigned index = 0; index < m.getSize (); index++) {
    const class PlaceMarking& place = m[index];
    buf.append (card_t (place.size ()), CARD_T_BIT);
    for (PlaceMarking::const_iterator i = place.begin ();
	 i != place.end (); i++) {
      assert (!place.getReservedCount (i));
      assert (place.getCount (i));
      buf.append (place.getCount (i), CARD_T_BIT);
      buf.append (place.getValue (i));
    }
  }
}

class GlobalMarking*
State::decode (const class Net& net, const word_t* data)
{
  class GlobalMarking* m = new class GlobalMarking (net);
  class BitUnpacker buf (data);
  for (unsigned i = 0; i < net.getNumPlaces (); i++) {
    for (unsigned token = buf.extract (CARD_T_BIT); token; token--) {
      card_t count = buf.extract (CARD_T_BIT);
      assert (count);
      (*m)[i].add (buf.extract (*net.getPlace (i)->getType ()), count);
    }
  }

  return m;
}
