// Arc class -*- c++ -*-

#ifndef ARC_H_
# define ARC_H_

class Arc {
public:
  // Constructors and destructors

  /** Constructs an Arc
   * @param output	Flag: is this an output arc?
   * @param expr	Arc expression (formal tokens)
   * @param place	Place associated with the arc
   * @param transition	Transition associated with the arc
   */
  Arc (bool output,
       class Marking* expr,
       const class Place* place);
private:
  /** Copy constructor */
  Arc (const class Arc& old);
  /** Assignment operator */
  class Arc& operator= (const class Arc& old);
public:
  /** Destructs an Arc */
  ~Arc ();

  // Accessors

  /** Determine whether this is an output arc */
  bool isOutput () const { return myIsOutput; }

  /** Get the arc expression */
  const class Marking* getExpr () const { return myExpr; }

  /** Get the place associated with this arc */
  const class Place* getPlace () const { return myPlace; }

  // Other methods

  /** Append an expression to the arc
   * @param expr       expression to be appended
   */
  void append (class Marking* expr);

  // Members

private:
  /** Flag: is this an output arc? */
  bool myIsOutput;
  /** Arc expression (formal tokens) */
  class Marking* myExpr;
  /** Place associated with the Arc */
  const class Place* myPlace;
};

#endif // ARC_H_
