// Place class -*- c++ -*-

#include "Place.h"
#include "Arc.h"
#include "Marking.h"

Place::Place (unsigned index, char* name, const class Type* type) :
  myIndex (index), myName (name), myType (type), myInitMarking (0),
  myNumInputs (0), myInputs (0), myNumOutputs (0), myOutputs (0)
{
}

Place::~Place ()
{
  delete[] myName;
  myInitMarking->destroy ();

  while (myNumInputs)
    delete myInputs[--myNumInputs];
  delete myInputs;

  while (myNumOutputs)
    delete myOutputs[--myNumOutputs];
  delete myOutputs;
}

#ifdef DEBUG
# include "util.h"
# include "Type.h"

void
Place::prettyPrint (unsigned indent) const
{
  print (indent, "place %s %s\n", myName,
	 myType->getName ()
	 ? myType->getName ()
	 : myType->getSyntacticName ());

  if (myInitMarking) {
    print (indent, ":\n");
    myInitMarking->prettyPrint (indent + 1);
  }
}

#endif // DEBUG
