// Place class -*- c++ -*-

#ifndef PLACE_H_
# define PLACE_H_

# include <cassert>

/** Place class */
class Place {
  /** The constructor can only be called by Net */
  friend class Net;
public:

protected:
  // Constructors and destructors

  /** Constructs a Place
   * @param index	Index number of the place (ask Net)
   * @param name	Name of the place
   * @param type	Type of the place
   */
  Place (unsigned index, char* name, const class Type* type);
private:
  /** Copy constructor */
  Place (const Place& old);
  /** Assignment operator */
  Place& operator= (const Place& old);
public:
  /** Destructs a Place */
  ~Place ();

  // Accessors

  /** Get the index number of the place */
  unsigned getIndex () const { return myIndex; }
  /** Get the name of the place */
  const char* getName () const { return myName; }
  /** Get the type of the place */
  const class Type* getType () const { return myType; }

  /** Get the number of input arcs */
  unsigned getNumInputs () const { return myNumInputs; }
  /** Get an input arc
   * @param i	Index number of the input arc
   */
  const class Arc* getInput (unsigned i) const {
    assert(i < myNumInputs); return myInputs[i];
  }

  /** Get the number of output arcs */
  unsigned getNumOutputs () const { return myNumOutputs; }
  /** Get an output arc
   * @param i	Index number of the output arc
   */
  const class Arc* getOutput (unsigned i) const {
    assert(i < myNumOutputs); return myOutputs[i];
  }

  /** Get the initial marking expression */
  const class Marking* getInitMarking () const { return myInitMarking; }
  /** Set the initial marking expression */
  void setInitMarking (class Marking* initMarking) {
    assert (!myInitMarking);
    myInitMarking = initMarking;
  }

  // Other methods

  /** Add an input arc
   * @param arc		The input arc to be added
   */
  void addInput (class Arc* arc);

  /** Add an output arc
   * @param arc		The output arc to be added
   */
  void addOutput (class Arc* arc);

# ifdef DEBUG
  /**
   * Prints this Place.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

  // Members

private:
  /** Index number of the place */
  unsigned myIndex;
  /** Name of the place */
  char* myName;
  /** Type of the place */
  const class Type* myType;
  /** Initial marking of the place */
  class Marking* myInitMarking;
  /** Number of input arcs */
  unsigned myNumInputs;
  /** Input arcs */
  class Arc** myInputs;
  /** Number of output arcs */
  unsigned myNumOutputs;
  /** Output arcs */
  class Arc** myOutputs;
};

#endif // PLACE_H_
