// Place marking class -*- c++ -*-

#include "PlaceMarking.h"
#include "Place.h"
#ifndef NDEBUG
# include "Type.h"
#endif
#include <cassert>

#ifndef NDEBUG
void
PlaceMarking::setPlace (const class Place* place)
{
  assert (!myPlace && place);
  setType (place->getType ());
  myPlace = place;
}

void
PlaceMarking::setType (const class Type* type)
{
  assert (!myPlace && !myType && type);
  myType = type;
}
#endif // NDEBUG

void
PlaceMarking::add (class Value* value, card_t amount)
{
  assert (value && value->getType ());
  assert (value->getType () == myType);
  assert (myType->isConstrained (value));
  assert (amount > 0);
  TokenMap::iterator i = myTokens.find (value);
  if (i == myTokens.end ()) {
    class card_pair p (amount, 0);
    myTokens.insert (std::pair<class Value*, class card_pair> (value, p));
  }
  else {
    assert (i->second.total >= i->second.reserved);
    i->second.total += amount;
    delete value;
  }
}

void
PlaceMarking::remove (const class Value* value, card_t amount)
{
  assert (value && value->getType ());
  assert (myType->isConstrained (value));
  assert (amount > 0);
  TokenMap::iterator i = find (value);
  assert (i != end ());
  assert (i->second.total >= i->second.reserved + amount);
  if (!(i->second.total -= amount)) {
    delete i->first;
    myTokens.erase (i);
  }
}

#include "Printer.h"
void
PlaceMarking::display (const class Printer& printer) const
{
  printer.print (myPlace->getName ());
  printer.delimiter (':')++;
  for (TokenMap::const_iterator i = myTokens.begin ();
       i != myTokens.end (); i++) {
    assert (!i->second.reserved);
    assert (i->second.total > 0);

    if (i == myTokens.begin ())
      printer.linebreak ();
    else
      printer.delimiter (',');

    if (i->second.total > 1) {
      printer.print (i->second.total);
      printer.delimiter ('#');
    }
    static_cast<const class Value*>(i->first)->display (printer);
  }
  printer--;
}

#ifdef DEBUG
# include "util.h"

void
PlaceMarking::prettyPrint (unsigned indent) const
{
# ifndef NDEBUG
  if (myPlace)
    print (indent, "%s:\n", myPlace->getName ());
# endif // NDEBUG
  for (TokenMap::const_iterator i = myTokens.begin ();
       i != myTokens.end (); i++) {
    print (indent + 1, "(total: %u, reserved: %u)\n",
	   i->second.total,
	   i->second.reserved);
    static_cast<const class Value*>(i->first)->prettyPrint (indent + 1);
  }
}

#endif // DEBUG
