// Boolean type class -*- c++ -*-

#include "BoolType.h"
#include "BoolValue.h"
#include "Constraint.h"
#include "Range.h"

BoolType::BoolType () :
  Type ()
{
}

BoolType::BoolType (const class BoolType& old) :
  Type ()
{
  if (old.getConstraint ())
    setConstraint (new class Constraint (*old.getConstraint (), this));
}

BoolType::~BoolType ()
{
}

class Value*
BoolType::getFirstValue () const
{
  const class Value* v = myConstraint ? myConstraint->getFirstValue () : NULL;
  assert (!v || v->getType () == this);
  return v ? v->copy () : new class BoolValue (this, false);
}

class Value*
BoolType::getLastValue () const
{
  const class Value* v = myConstraint ? myConstraint->getLastValue () : NULL;
  assert (!v || v->getType () == this);
  return v ? v->copy () : new class BoolValue (this, true);
}

card_t
BoolType::do_getNumValues () const
{
  if (!myConstraint)
    return 2;
  card_t numValues = 0;

  for (Constraint::const_iterator i = myConstraint->begin ();
       !(i == myConstraint->end ()); i++) {
    const class Range* r = *i;
    bool lowLimit = false;
    bool highLimit = true;

    if (const class Value* low = r->getLow ())
      lowLimit = static_cast<const class BoolValue*>(low)->getValue ();
    if (const class Value* high = r->getHigh ())
      highLimit = static_cast<const class BoolValue*>(high)->getValue ();
    assert (lowLimit <= highLimit);

    card_t diff = highLimit - lowLimit + 1;
    assert (diff == 1 || diff == 2);
    numValues += diff;
  }

  return numValues;
}

card_t
BoolType::convert (const class Value* value) const
{
  assert (value && value->getKind () == Value::vBool);
  bool v = static_cast<const class BoolValue*>(value)->getValue ();
  if (!myConstraint)
    return v ? 1 : 0;
  else {
    assert (isConstrained (value));
    card_t number = 0;

    for (Constraint::const_iterator i = myConstraint->begin ();
	 !(i == myConstraint->end ()); i++) {
      const class Range* r = *i;
      bool lowLimit = false;
      bool highLimit = true;

      if (const class Value* low = r->getLow ())
	lowLimit = static_cast<const class BoolValue*>(low)->getValue ();
      if (const class Value* high = r->getHigh ())
	highLimit = static_cast<const class BoolValue*>(high)->getValue ();
      assert (lowLimit <= highLimit);

      if (v < lowLimit || v > highLimit) {
	card_t diff = highLimit - lowLimit + 1;
	assert (diff == 1 || diff == 2);
	number += diff;
      }
      else {
	number += v - lowLimit;
	assert (number <= getNumValues ());
	return number;
      }
    }

    assert (false);
    return CARD_T_MAX;
  }
}

class Value*
BoolType::convert (card_t number) const
{
  assert (number < getNumValues ());
  if (!myConstraint)
    return new class BoolValue (this, number ? true : false);
  else {
    for (Constraint::const_iterator i = myConstraint->begin ();
	 !(i == myConstraint->end ()); i++) {
      const class Range* r = *i;
      bool lowLimit = false;
      bool highLimit = true;

      if (const class Value* low = r->getLow ())
	lowLimit = static_cast<const class BoolValue*>(low)->getValue ();
      if (const class Value* high = r->getHigh ())
	highLimit = static_cast<const class BoolValue*>(high)->getValue ();
      assert (lowLimit <= highLimit);

      card_t diff = highLimit - lowLimit;
      if (number <= diff)
	return new class BoolValue (this, lowLimit + number ? true : false);
      else
	number -= diff, number--;
    }

    assert (false);
    return NULL;
  }
}
