// Boolean type class -*- c++ -*-

#ifndef BOOLTYPE_H_
# define BOOLTYPE_H_

# include "Type.h"

/** Boolean type */
class BoolType : public Type {
public:
  // Constructors and destructors

  /** Constructs BoolType */
  BoolType ();
private:
  /** Copy constructor */
  BoolType (const class BoolType& old);
  /** Assignment operator */
  class BoolType& operator= (const class BoolType& old);
public:
  /** Destructs BoolType */
  ~BoolType ();
  /** Copy the Type */
  class Type* copy () const { return new class BoolType (*this); }

  // Accessors

  /** Determine the kind of the type */
  enum Kind getKind () const { return tBool; }

  /** Determine whether the type is ordered */
  bool isOrdered () const { return true; }
  /** Get the first value of this type */
  class Value* getFirstValue () const;
  /** Get the last value of this type */
  class Value* getLastValue () const;
  /** Get the number of possible values for this type. */
  card_t do_getNumValues () const;

  // Other methods

  /** Convert a value of this type to a number
   * @param value	value to be converted
   * @return		number between 0 and getNumValues () - 1
   */
  card_t convert (const class Value* value) const;

  /** Convert a number to a value of this type
   * @param number	number between 0 and getNumValues () - 1
   * @return		the corresponding value
   */
  class Value* convert (card_t number) const;
};

#endif // BOOLTYPE_H_
