// Character type class -*- c++ -*-

#include "CharType.h"
#include "CharValue.h"
#include "Constraint.h"
#include "Range.h"

CharType::CharType () :
  Type ()
{
}

CharType::CharType (const class CharType& old) :
  Type ()
{
  if (old.getConstraint ())
    setConstraint (new class Constraint (*old.getConstraint (), this));
}

CharType::~CharType ()
{
}

class Value*
CharType::getFirstValue () const
{
  const class Value* v = myConstraint ? myConstraint->getFirstValue () : NULL;
  assert (!v || v->getType () == this);
  return v ? v->copy () : new class CharValue (this, CHAR_T_MIN);
}

class Value*
CharType::getLastValue () const
{
  const class Value* v = myConstraint ? myConstraint->getLastValue () : NULL;
  assert (!v || v->getType () == this);
  return v ? v->copy () : new class CharValue (this, CHAR_T_MAX);
}

card_t
CharType::do_getNumValues () const
{
  if (!myConstraint)
    return
      static_cast<card_t>(static_cast<int_t>(CHAR_T_MAX) - CHAR_T_MIN + 1);
  card_t numValues = 0;

  for (Constraint::const_iterator i = myConstraint->begin ();
       !(i == myConstraint->end ()); i++) {
    const class Range* r = *i;
    char_t lowLimit = CHAR_T_MIN;
    char_t highLimit = CHAR_T_MAX;

    if (const class Value* low = r->getLow ())
      lowLimit = static_cast<const class CharValue*>(low)->getValue ();
    if (const class Value* high = r->getHigh ())
      highLimit = static_cast<const class CharValue*>(high)->getValue ();
    assert (lowLimit <= highLimit);

    numValues += static_cast<int_t>(highLimit) - lowLimit + 1;
    assert (numValues <= static_cast<int_t>(CHAR_T_MAX) - CHAR_T_MIN + 1);
  }

  return numValues;
}

card_t
CharType::convert (const class Value* value) const
{
  assert (value && value->getKind () == Value::vChar);
  char_t v = static_cast<const class CharValue*>(value)->getValue ();
  if (!myConstraint)
    return static_cast<card_t>(static_cast<unsigned char>(v));
  else {
    assert (isConstrained (value));
    card_t number = 0;

    for (Constraint::const_iterator i = myConstraint->begin ();
	 !(i == myConstraint->end ()); i++) {
      const class Range* r = *i;
      char_t lowLimit = CHAR_T_MIN;
      char_t highLimit = CHAR_T_MAX;

      if (const class Value* low = r->getLow ())
	lowLimit = static_cast<const class CharValue*>(low)->getValue ();
      if (const class Value* high = r->getHigh ())
	highLimit = static_cast<const class CharValue*>(high)->getValue ();
      assert (lowLimit <= highLimit);

      if (v < lowLimit || v > highLimit)
	number += static_cast<int_t>(highLimit) - lowLimit + 1;
      else {
	number += static_cast<int_t>(v) - lowLimit;
	assert (number <= getNumValues ());
	return number;
      }
    }

    assert (false);
    return CARD_T_MAX;
  }
}

class Value*
CharType::convert (card_t number) const
{
  assert (number < getNumValues ());
  if (!myConstraint)
    return new class CharValue (this, static_cast<char_t>(number));
  else {
    for (Constraint::const_iterator i = myConstraint->begin ();
	 !(i == myConstraint->end ()); i++) {
      const class Range* r = *i;
      char_t lowLimit = CHAR_T_MIN;
      char_t highLimit = CHAR_T_MAX;

      if (const class Value* low = r->getLow ())
	lowLimit = static_cast<const class CharValue*>(low)->getValue ();
      if (const class Value* high = r->getHigh ())
	highLimit = static_cast<const class CharValue*>(high)->getValue ();
      assert (lowLimit <= highLimit);

      card_t diff = static_cast<int_t>(highLimit) - lowLimit;
      if (number <= diff)
	return new class CharValue (this, lowLimit + number);
      else
	number -= diff, number--;
    }

    assert (false);
    return NULL;
  }
}
