// Character type class -*- c++ -*-

#ifndef CHARTYPE_H_
# define CHARTYPE_H_

# include "Type.h"

/** Boolean type */
class CharType : public Type {
public:
  // Constructors and destructors

  /** Constructs CharType */
  CharType ();
private:
  /** Copy constructor */
  CharType (const class CharType& old);
  /** Assignment operator */
  class CharType& operator= (const class CharType& old);
public:
  /** Destructs CharType */
  ~CharType ();
  /** Copy the Type */
  class Type* copy () const { return new class CharType (*this); }

  // Accessors

  /** Determine the kind of the type */
  enum Kind getKind () const { return tChar; }

  /** Determine whether the type is ordered */
  bool isOrdered () const { return true; }
  /** Get the first value of this type */
  class Value* getFirstValue () const;
  /** Get the last value of this type */
  class Value* getLastValue () const;
  /** Determine the number of possible values for this type */
  card_t do_getNumValues () const;

  // Other methods

  /** Convert a value of this type to a number
   * @param value	value to be converted
   * @return		number between 0 and getNumValues () - 1
   */
  card_t convert (const class Value* value) const;

  /** Convert a number to a value of this type
   * @param number	number between 0 and getNumValues () - 1
   * @return		the corresponding value
   */
  class Value* convert (card_t number) const;
};

#endif // CHARTYPE_H_
