// Identifier type class -*- c++ -*-

#ifndef IDTYPE_H_
# define IDTYPE_H_

# include "Type.h"

/** Boolean type */
class IdType : public Type {
public:
  // Constructors and destructors

  /** Constructs IdType
   * @param size	Number of values in the identifier pool
   */
  IdType (card_t size);
private:
  /** Copy constructor */
  IdType (const class IdType& old);
  /** Assignment operator */
  class IdType& operator= (const class IdType& old);
public:
  /** Destructs IdType */
  ~IdType ();
  /** Copy the Type */
  class Type* copy () const { return new class IdType (*this); }

  // Accessors

  /** Determine the kind of the type */
  enum Kind getKind () const { return tId; }

  /** Determine whether the type is ordered */
  bool isOrdered () const { return false; }
  /** Get the first value of this type */
  class Value* getFirstValue () const;
  /** Get the last value of this type */
  class Value* getLastValue () const;

  /** Determine the number of values in the identifier pool */
  card_t getSize () const { return mySize; }

  // Other methods

  /** Convert a value of this type to a number
   * @param value	value to be converted
   * @return		number between 0 and getNumValues () - 1
   */
  card_t convert (const class Value* value) const;

  /** Convert a number to a value of this type
   * @param number	number between 0 and getNumValues () - 1
   * @return		the corresponding value
   */
  class Value* convert (card_t number) const;

  /** See if the type is assignable to another type
   * @param type	the type this type should be assignable to
   * @return		true if this type is assignable to the type
   */
  bool isAssignable (const class Type* type) const { return type == this; }

  /** Get the number of possible values for this type */
  card_t do_getNumValues () const;

private:
  /** Number of values in the identifier pool */
  const card_t mySize;
};

#endif // IDTYPE_H_
