// Integer type class -*- c++ -*-

#include "IntType.h"
#include "IntValue.h"
#include "Constraint.h"
#include "Range.h"

IntType::IntType () :
  Type ()
{
}

IntType::IntType (const class IntType& old) :
  Type ()
{
  if (old.getConstraint ())
    setConstraint (new class Constraint (*old.getConstraint (), this));
}

IntType::~IntType ()
{
}

class Value*
IntType::getFirstValue () const
{
  const class Value* v = myConstraint ? myConstraint->getFirstValue () : NULL;
  assert (!v || v->getType () == this);
  return v ? v->copy () : new class IntValue (this, INT_T_MIN);
}

class Value*
IntType::getLastValue () const
{
  const class Value* v = myConstraint ? myConstraint->getLastValue () : NULL;
  assert (!v || v->getType () == this);
  return v ? v->copy () : new class IntValue (this, INT_T_MAX);
}

card_t
IntType::do_getNumValues () const
{
  if (!myConstraint)
    return CARD_T_MAX;
  card_t numValues = 0;

  for (Constraint::const_iterator i = myConstraint->begin ();
       !(i == myConstraint->end ()); i++) {
    const class Range* r = *i;
    int_t lowLimit = INT_T_MIN;
    int_t highLimit = INT_T_MAX;

    if (const class Value* low = r->getLow ())
      lowLimit = static_cast<const class IntValue*>(low)->getValue ();
    if (const class Value* high = r->getHigh ())
      highLimit = static_cast<const class IntValue*>(high)->getValue ();
    assert (lowLimit <= highLimit);

    if (int_t diff = highLimit - lowLimit + 1) {
      assert (CARD_T_MAX - numValues >= static_cast<card_t>(diff));
      numValues += static_cast<card_t>(diff);
    }
    else { // overflow
      assert (!numValues);
      return CARD_T_MAX;
    }
  }

  return numValues;
}

card_t
IntType::convert (const class Value* value) const
{
  assert (value && value->getKind () == Value::vInt);
  int_t v = static_cast<const class IntValue*>(value)->getValue ();
  if (!myConstraint)
    return static_cast<card_t>(v);
  else {
    assert (isConstrained (value));
    card_t number = 0;

    for (Constraint::const_iterator i = myConstraint->begin ();
	 !(i == myConstraint->end ()); i++) {
      const class Range* r = *i;
      int_t lowLimit = INT_T_MIN;
      int_t highLimit = INT_T_MAX;

      if (const class Value* low = r->getLow ())
	lowLimit = static_cast<const class IntValue*>(low)->getValue ();
      if (const class Value* high = r->getHigh ())
	highLimit = static_cast<const class IntValue*>(high)->getValue ();
      assert (lowLimit <= highLimit);

      if (v < lowLimit || v > highLimit) {
	if (int_t diff = highLimit - lowLimit + 1)
	  number += diff;
	else {
	  assert (false);
	  return CARD_T_MAX;
	}
      }
      else {
	number += static_cast<int_t>(v) - lowLimit;
	assert (number <= getNumValues ());
	return number;
      }
    }

    assert (false);
    return CARD_T_MAX;
  }
}

class Value*
IntType::convert (card_t number) const
{
  assert (number < getNumValues ());
  if (!myConstraint)
    return new class IntValue (this, static_cast<int_t>(number));
  else {
    for (Constraint::const_iterator i = myConstraint->begin ();
	 !(i == myConstraint->end ()); i++) {
      const class Range* r = *i;
      int_t lowLimit = INT_T_MIN;
      int_t highLimit = INT_T_MAX;

      if (const class Value* low = r->getLow ())
	lowLimit = static_cast<const class IntValue*>(low)->getValue ();
      if (const class Value* high = r->getHigh ())
	highLimit = static_cast<const class IntValue*>(high)->getValue ();
      assert (lowLimit <= highLimit);

      card_t diff = highLimit - lowLimit;
      if (number <= diff)
	return new class IntValue (this, lowLimit + number);
      else
	number -= diff, number--;
    }

    assert (false);
    return NULL;
  }
}
