// Maria range class -*- c++ -*-

#include "Range.h"
#include "Value.h"

Range::Range (class Value* low, class Value* high) :
  myLow (low), myHigh (high)
{
  assert (!myLow || myLow->getType ());
  assert (!myHigh || myHigh->getType ());
  assert (!myLow || !myHigh || myLow->getType () == myHigh->getType ());

  if (myLow && myHigh && *myHigh < *myLow) {
    class Value* v = myLow;
    myLow = myHigh;
    myHigh = v;
  }
}

Range::Range (const class Range& old) :
  myLow (old.myLow ? old.myLow->copy () : 0),
  myHigh (old.myHigh ? old.myHigh->copy () : 0)
{
  assert (!myLow || myLow->getType ());
  assert (!myHigh || myHigh->getType ());
  assert (!myLow || !myHigh || myLow->getType () == myHigh->getType ());
  assert (!myLow || !myHigh || *myLow <= *myHigh);
}

Range::~Range ()
{
  delete myLow;
  if (myLow != myHigh)
    delete myHigh;
}

void
Range::setType (const class Type* type)
{
  assert (type != 0);
  if (myLow)
    myLow->setType (type);
  if (myHigh)
    myHigh->setType (type);
}

class Range*
Range::combine (const class Range* other) const
{
  assert (!!other);

  if (myLow && myHigh) {
    if (other->myLow && other->myHigh) {
      if (*other->myLow <= *myLow && *myHigh <= *other->myHigh)
	return new class Range (other->myLow->copy (), other->myHigh->copy ());
      if (*other->myLow <= *myLow && *myLow <= *other->myHigh)
	return new class Range (other->myLow->copy (), myHigh->copy ());
      if (*other->myLow <= *myHigh && *myHigh <= *other->myHigh)
	return new class Range (myLow->copy (), other->myHigh->copy ());
      if (*myLow <= *other->myLow && *other->myHigh <= *myHigh)
	return new class Range (myLow->copy (), myHigh->copy ());
    }
    else if (other->myLow) {
      if (*other->myLow <= *myLow)
	return new class Range (other->myLow->copy (), 0);
      if (*other->myLow <= *myHigh)
	return new class Range (myLow->copy (), 0);
    }
    else if (other->myHigh) {
      if (*myHigh <= *other->myHigh)
	return new class Range (0, other->myHigh->copy ());
      if (*myLow <= *other->myHigh)
	return new class Range (0, myHigh->copy ());
    }
    else
      return new class Range (0, 0);
  }
  else if (myLow) {
    if (other->myLow) {
      if (*myLow <= *other->myLow)
	return new class Range (myLow->copy (), 0);
      if (!other->myHigh || *myLow <= *other->myHigh)
	return new class Range (other->myLow->copy (), 0);
    }
    else if (!other->myHigh || *myLow <= *other->myHigh)
      return new class Range (0, 0);
  }
  else if (myHigh) {
    if (other->myHigh) {
      if (*other->myHigh <= *myHigh)
	return new class Range (0, myHigh->copy ());
      if (!other->myLow || *other->myLow <= *myHigh)
	return new class Range (0, other->myHigh->copy ());
    }
    else if (!other->myLow || *other->myLow <= *myHigh)
      return new class Range (0, 0);
  }
  else
    return new class Range (0, 0);

  if (myHigh && other->myLow) {
    class Value* v = myHigh->copy ();
    v->increment ();
    if (*v == *other->myLow) {
      delete v;
      return new class Range (myLow->copy (), other->myHigh->copy ());
    }
    delete v;
  }

  if (myLow && other->myHigh) {
    class Value* v = other->myHigh->copy ();
    v->increment ();
    if (*v == *myLow) {
      delete v;
      return new class Range (other->myLow->copy (), myHigh->copy ());
    }
    delete v;
  }

  return 0;
}

class Range*
Range::cut (const class Range* other) const
{
  assert (!!other);

  if (myLow && myHigh) {
    if (other->myLow && other->myHigh) {
      if (*other->myLow <= *myLow && *myHigh <= *other->myHigh)
	return new class Range (myLow->copy (), myHigh->copy ());
      if (*other->myLow <= *myLow && *myLow <= *other->myHigh)
	return new class Range (myLow->copy (), other->myHigh->copy ());
      if (*other->myLow <= *myHigh && *myHigh <= *other->myHigh)
	return new class Range (other->myLow->copy (), myHigh->copy ());
      if (*myLow <= *other->myLow && *other->myHigh <= *myHigh)
	return new class Range (other->myLow->copy (), other->myHigh->copy ());
    }
    else if (other->myLow) {
      if (*other->myLow <= *myLow)
	return new class Range (myLow->copy (), myHigh->copy ());
      if (*other->myLow <= *myHigh)
	return new class Range (other->myLow->copy (), myHigh->copy ());
    }
    else if (other->myHigh) {
      if (*myHigh <= *other->myHigh)
	return new class Range (myLow->copy (), myHigh->copy ());
      if (*myLow <= *other->myHigh)
	return new class Range (myLow->copy (), other->myHigh->copy ());
    }
    else
      return new class Range (myLow->copy (), myHigh->copy ());
  }
  else if (myLow) {
    if (other->myLow) {
      if (*myLow <= *other->myLow)
	return new class Range (other->myLow->copy (),
				other->myHigh ? other->myHigh->copy () : 0);
      if (!other->myHigh || *myLow <= *other->myHigh)
	return new class Range (myLow->copy (),
				other->myHigh ? other->myHigh->copy () : 0);
    }
    else if (!other->myHigh || *myLow <= *other->myHigh)
      return new class Range (myLow->copy (),
			      other->myHigh ? other->myHigh->copy () : 0);
  }
  else if (myHigh) {
    if (other->myHigh) {
      if (*other->myHigh <= *myHigh)
	return new class Range (other->myLow ? other->myLow->copy () : 0,
				other->myHigh->copy ());
      if (!other->myLow || *other->myLow <= *myHigh)
	return new class Range (other->myLow ? other->myLow->copy () : 0,
				myHigh->copy ());
    }
    else if (!other->myLow || *other->myLow <= *myHigh)
      return new class Range (other->myLow ? other->myLow->copy () : 0,
			      myHigh->copy ());
  }
  else
    return new class Range (other->myLow ? other->myLow->copy () : 0,
			    other->myHigh ? other->myHigh->copy () : 0);

  return 0;
}

enum Range::Result
Range::check (const class Value* value) const
{
  if (!value)
    return undefined;

  if (myLow == myHigh)
    return *value == *myLow ? pass : fail;

  if (myLow && *value < *myLow)
    return fail;
  if (myHigh && *myHigh < *value)
    return fail;
  return pass;
}

#ifdef DEBUG
# include "util.h"

void
Range::prettyPrint (unsigned indent) const
{
  if (myLow)
    myLow->prettyPrint (indent);
  if (myLow != myHigh) {
    print (indent, "..\n");
    if (myHigh)
      myHigh->prettyPrint (indent);
  }
}

#endif // DEBUG
