// Maria range class -*- c++ -*-

#ifndef RANGE_H_
# define RANGE_H_

/** Constant range (open or closed) */
class Range {
public:
  /** Result of range check */
  enum Result { undefined, pass, fail };

  // Constructors and destructors

  /** Constructs a new Range
   * @param low		Lower bound of the range (NULL if open)
   * @param high	Upper bound of the range (NULL if open)
   */
  Range (class Value* low, class Value* high);
  /** Copy constructor */
  Range (const class Range& old);
private:
  /** Assignment operator */
  class Range& operator= (const class Range& old);
public:
  /** Destructs Range */
  ~Range ();

  // Accessors

  /** Get the lower bound */
  const class Value* getLow () const { return myLow; }
  /** Get the higher bound */
  const class Value* getHigh () const { return myHigh; }

  /** Set the type of the Range
   * @param type	type the limits of the range are to have
   */
  void setType (const class Type* type);

  // Other methods

  /** Combine the two ranges if possible
   * @return		pointer to the union of both ranges, or NULL
   */
  class Range* combine (const class Range* other) const;
  /** Determine the intersection of the two ranges
   * @return		pointer to the intersection of both ranges, or NULL
   */
  class Range* cut (const class Range* other) const;

  /** Check whether the value passes the range constraint
   * @param value	Value to be checked
   * @return		(@see Result)
   */
  enum Result check (const class Value* value) const;

# ifdef DEBUG
  /**
   * Prints this object.
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** Lower bound */
  class Value* myLow;
  /** Higher bound */
  class Value* myHigh;
};

#endif // RANGE_H_
