// Data type base class -*- c++ -*-

#include "UnionType.h"
#include "Constraint.h"
#ifndef NDEBUG
# include "Value.h"
#endif // NDEBUG

Type::Type () :
  myNumValues (0), myName (0), myConstraint (0)
{
}

Type::~Type ()
{
  delete[] myName;
  delete myConstraint;
}

const char*
Type::getSyntacticName () const
{
  switch (getKind ()) {
  case tInt:
    return "int";
  case tBool:
    return "bool";
  case tChar:
    return "char";
  case tEnum:
    return "enum";
  case tId:
    return "id";
  case tStruct:
    return "struct";
  case tVector:
    return "vector";
  case tUnion:
    return "union";
  default:
    return "";
  }
}

bool
Type::isAssignable (const class Type* type) const
{
  assert (type != NULL);
  if (type->getKind () == getKind ())
    return
      (!type->getConstraint () || !getConstraint () ||
       !type->getConstraint ()->isDisjoint (getConstraint ()));
  else
    return type->getKind () == tUnion &&
      static_cast<const class UnionType*>(type)->isAssignableFrom (*this);
}

bool
Type::isAlwaysAssignable (const class Type* type) const
{
  assert (type != NULL);
  if (type->getKind () == getKind ())
    return isAssignable (type) &&
      (!type->getConstraint () ||
       (getConstraint () &&
	type->getConstraint ()->isSuperset (getConstraint ())));
  else
    return type->getKind () == tUnion &&
      static_cast<const class UnionType*>(type)
      ->isAlwaysAssignableFrom (*this);
}

bool
Type::isConstrained (const class Value* value) const
{
  assert (value && value->getType ());
  assert (value->getType ()->isAssignable (this));
  return !getConstraint () || getConstraint ()->isConstrained (value);
}

void
Type::setConstraint (class Constraint* constraint)
{
  assert (!myConstraint && constraint);
  (myConstraint = constraint)->setType (this);
}

#ifdef DEBUG
# include "util.h"
void
Type::prettyPrint (unsigned indent) const
{
  print (indent, "%s\n", getSyntacticName ());
  if (myConstraint)
    myConstraint->prettyPrint (indent);
}
#endif // DEBUG
