// Boolean value class -*- c++ -*-

#include "BoolValue.h"
#include "Type.h"
#include "Constraint.h"

BoolValue::BoolValue (const class Type* type, bool value) :
  Value (type),
  myValue (value)
{
  assert (getType ()->getKind () == Type::tBool);
}

BoolValue::BoolValue (const class Type* type) :
  Value (type),
  myValue (false)
{
  assert (getType ()->getKind () == Type::tBool);
  bottom ();
}

BoolValue::BoolValue (const class BoolValue& old) :
  Value (old.getType ()),
  myValue (old.getValue ())
{
  assert (getType ()->getKind () == Type::tBool);
}

BoolValue::~BoolValue ()
{
}

void
BoolValue::bottom ()
{
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getFirstValue ();
    assert (!v || v->getKind () == getKind ());
    myValue = v ? static_cast<const class BoolValue*>(v)->myValue : false;
  }
  else
    myValue = false;
}

void
BoolValue::top ()
{
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getLastValue ();
    assert (!v || v->getKind () == getKind ());
    myValue = v ? static_cast<const class BoolValue*>(v)->myValue : true;
  }
  else
    myValue = false;
}

bool
BoolValue::increment ()
{
  if (getType ()->getConstraint ()) {
    if (myValue) {
      bottom ();
      return false;
    }
    else {
      top ();
      return myValue;
    }
  }

  return myValue = !myValue;
}

bool
BoolValue::decrement ()
{
  if (getType ()->getConstraint ()) {
    if (myValue) {
      bottom ();
      return !myValue;
    }
    else {
      top ();
      return false;
    }
  }

  return !(myValue = !myValue);
}

#include "Printer.h"

void
BoolValue::display (const class Printer& printer) const
{
  printer.print (myValue);
}

#ifdef DEBUG
# include "util.h"

void
BoolValue::prettyPrint (unsigned indent) const
{
  print (indent, "%s\n", myValue ? "true" : "false");
}

#endif // DEBUG
