// Boolean value class -*- c++ -*-

#ifndef BOOLVALUE_H_
# define BOOLVALUE_H_

# include "Value.h"

/** Boolean value */
class BoolValue : public Value {
public:
  // Constructors and destructors

  /** Constructs BoolValue
   * @param type	type of the value
   * @param value	the boolean value
   */
  BoolValue (const class Type* type, bool value);
  /** Constructs BoolValue, initializing it to the smallest possible value
   * @param type	type of the value
   */
  BoolValue (const class Type* type);
private:
  /** Copy constructor */
  BoolValue (const class BoolValue& old);
  /** Assignment operator */
  class BoolValue& operator= (const class BoolValue& old);
public:
  /** Destructs BoolValue */
  ~BoolValue ();
  /** Copies BoolValue */
  class Value* copy () const { return new class BoolValue (*this); }

  // Accessors

  /** Determine the kind of the value */
  enum Kind getKind () const { return vBool; }

  /** Determine the value */
  bool getValue () const { return myValue; }
  /** Set the value */
  void setValue (bool value) { myValue = value; }

  // Other methods

  /** Less-than comparison */
  bool operator< (const class BoolValue& other) const {
    return myValue < other.myValue;
  }
  /** Less-or-equal comparison */
  bool operator<= (const class BoolValue& other) const {
    return myValue <= other.myValue;
  }
  /** Equality comparison */
  bool operator== (const class BoolValue& other) const {
    return myValue == other.myValue;
  }

  /** Difference operator
   * @param other	value to be substracted from this (may not be less)
   * @return		the difference
   */
  card_t operator- (const class BoolValue& other) const {
    return myValue - other.myValue;
  }

  /** Reset the value to the first value */
  void bottom ();
  /** Reset the value to the last value */
  void top ();
  /** Get the next value
   * @return	false if the value wrapped around; otherwise true
   */
  bool increment ();
  /** Get the previous value
   * @return	false if the value wrapped around; otherwise true
   */
  bool decrement ();

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

# ifdef DEBUG
  /**
   * Print this object
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The value */
  bool myValue;
};

#endif // BOOLVALUE_H_
