// Buffer value class -*- c++ -*-

#ifndef BUFFERVALUE_H_
# define BUFFERVALUE_H_

# include "Value.h"
# include "ValueList.h"

/** Buffer value */
class BufferValue : public Value {
public:
  // Constructors and destructors

  /** Constructs BufferValue
   * @param type	type of the value
   */
  BufferValue (const class Type* type);
private:
  /** Copy constructor */
  BufferValue (const class BufferValue& old);
  /** Assignment operator */
  class BufferValue& operator= (const class BufferValue& old);
public:
  /** Destructs BufferValue */
  ~BufferValue ();
  /** Copies BufferValue */
  class Value* copy () const { return new class BufferValue (*this); }

  // Accessors

  /** Determine the kind of the value */
  enum Kind getKind () const { return vBuffer; }

  /** Determine the maximum number of components in the buffer */
  card_t getSize () const { return myComponents.getSize (); }
  /** Determine the current capacity of the buffer */
  card_t getCapacity () const {
    for (card_t i = getSize (); i; )
      if (myComponents[--i])
	return i + 1;
    return 0;
  }

  /** Address a component by index */
  const class Value* operator[] (card_t index) const {
    return myComponents[index];
  }
  /** Address a component by index */
  class Value*& operator[] (card_t index) {
    return myComponents[index];
  }

  // Other methods

  /** Less-than comparison */
  bool operator< (const class BufferValue& other) const;
  /** Less-or-equal comparison */
  bool operator<= (const class BufferValue& other) const;
  /** Equality comparison */
  bool operator== (const class BufferValue& other) const;

  /** Difference operator
   * @param other	value to be substracted from this (may not be less)
   * @return		the difference
   */
  card_t operator- (const class BufferValue& other) const;

  /** Reset the value to the first value */
  void bottom ();
  /** Reset the value to the last value */
  void top ();
  /** Get the next value
   * @return	false if the value wrapped around; otherwise true
   */
  bool increment ();
  /** Get the previous value
   * @return	false if the value wrapped around; otherwise true
   */
  bool decrement ();

  /** Convert the value to another type if possible
   * @param type	type to be casted to
   * @return		the converted value, or NULL
   */
  class Value* cast (const class Type& type);

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

# ifdef DEBUG
  /**
   * Print this object
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The components */
  class ValueList myComponents;
};

#endif // BUFFERVALUE_H_
