// Character value class -*- c++ -*-

#include "CharValue.h"
#include "Type.h"
#include "Constraint.h"

CharValue::CharValue (const class Type* type, char_t value) :
  Value (type),
  myValue (value)
{
  assert (getType ()->getKind () == Type::tChar);
}

CharValue::CharValue (const class Type* type) :
  Value (type),
  myValue (0)
{
  assert (getType ()->getKind () == Type::tChar);
  bottom ();
}

CharValue::CharValue (const class CharValue& old) :
  Value (old.getType ()),
  myValue (old.getValue ())
{
  assert (getType ()->getKind () == Type::tChar);
}

CharValue::~CharValue ()
{
}

void
CharValue::bottom ()
{
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getFirstValue ();
    assert (!v || v->getKind () == getKind ());
    myValue = v ? static_cast<const class CharValue*>(v)->myValue : CHAR_T_MIN;
  }
  else
    myValue = CHAR_T_MIN;
}

void
CharValue::top ()
{
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getLastValue ();
    assert (!v || v->getKind () == getKind ());
    myValue = v ? static_cast<const class CharValue*>(v)->myValue : CHAR_T_MAX;
  }
  else
    myValue = CHAR_T_MAX;
}

bool
CharValue::increment ()
{
  if (myValue == CHAR_T_MAX) {
    bottom ();
    return false;
  }

  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getNextHigh (this);
    if (v) {
      assert (v->getKind () == getKind ());
      if (myValue == static_cast<const class CharValue*>(v)->myValue) {
	v = getType ()->getConstraint ()->getNextLow (this);
	if (!v) {
	  bottom ();
	  return false;
	}
	assert (v->getKind () == getKind ());
	myValue = static_cast<const class CharValue*>(v)->myValue;
	return true;
      }
    }
  }

  myValue++;
  return true;
}

bool
CharValue::decrement ()
{
  if (myValue == CHAR_T_MAX) {
    top ();
    return false;
  }

  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getPrevLow (this);
    if (v) {
      assert (v->getKind () == getKind ());
      if (myValue == static_cast<const class CharValue*>(v)->myValue) {
	v = getType ()->getConstraint ()->getPrevHigh (this);
	if (!v) {
	  top ();
	  return false;
	}
	assert (v->getKind () == getKind ());
	myValue = static_cast<const class CharValue*>(v)->myValue;
	return true;
      }
    }
  }

  myValue--;
  return true;
}

#include "Printer.h"

void
CharValue::display (const class Printer& printer) const
{
  printer.print (myValue);
}

#ifdef DEBUG
# include "util.h"
# include <cctype>

void
CharValue::prettyPrint (unsigned indent) const
{
  switch (myValue) {
  case '\f':
    print (indent, "'\\f'\n");
    break;
  case '\n':
    print (indent, "'\\n'\n");
    break;
  case '\r':
    print (indent, "'\\r'\n");
    break;
  case '\t':
    print (indent, "'\\t'\n");
    break;
  case '\v':
    print (indent, "'\\v'\n");
    break;
  case '\b':
    print (indent, "'\\b'\n");
    break;
  case '\a':
    print (indent, "'\\a'\n");
    break;
  case '\'':
    print (indent, "'\\''\n");
    break;
  default:
    if (iscntrl (static_cast<unsigned char>(myValue)))
      print (indent, "'\\%o'\n", static_cast<unsigned char>(myValue));
    else
      print (indent, "'%c'\n", myValue);
    break;
  }
}

#endif // DEBUG
