// Character value class -*- c++ -*-

#ifndef CHARVALUE_H_
# define CHARVALUE_H_

# include "Value.h"
# include "typedefs.h"

/** Character value */
class CharValue : public Value {
public:
  // Constructors and destructors

  /** Constructs CharValue
   * @param type	type of the value
   * @param value	the character value
   */
  CharValue (const class Type* type, char_t value);
  /** Constructs CharValue, initializing it to the smallest possible value
   * @param type	type of the value
   */
  CharValue (const class Type* type);
private:
  /** Copy constructor */
  CharValue (const class CharValue& old);
  /** Assignment operator */
  class CharValue& operator= (const class CharValue& old);
public:
  /** Destructs CharValue */
  ~CharValue ();
  /** Copies CharValue */
  class Value* copy () const { return new class CharValue (*this); }

  // Accessors

  /** Determine the kind of the value */
  enum Kind getKind () const { return vChar; }

  /** Determine the value */
  char_t getValue () const { return myValue; }

  // Other methods

  /** Less-than comparison */
  bool operator< (const class CharValue& other) const {
    return myValue < other.myValue;
  }
  /** Less-or-equal comparison */
  bool operator<= (const class CharValue& other) const {
    return myValue <= other.myValue;
  }
  /** Equality comparison */
  bool operator== (const class CharValue& other) const {
    return myValue == other.myValue;
  }

  /** Difference operator
   * @param other	value to be substracted from this (may not be less)
   * @return		the difference
   */
  card_t operator- (const class CharValue& other) const {
    return myValue - other.myValue;
  }

  /** Reset the value to the first value */
  void bottom ();
  /** Reset the value to the last value */
  void top ();
  /** Get the next value
   * @return	false if the value wrapped around; otherwise true
   */
  bool increment ();
  /** Get the previous value
   * @return	false if the value wrapped around; otherwise true
   */
  bool decrement ();

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

# ifdef DEBUG
  /**
   * Print this object
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The value */
  char_t myValue;
};

#endif // CHARVALUE_H_
