// Enumeration value class -*- c++ -*-

#include "EnumValue.h"
#include "EnumType.h"
#include "Constraint.h"

EnumValue::EnumValue (const class Type* type, int_t value) :
  Value (type),
  myValue (value)
{
  assert (getType ()->getKind () == Type::tEnum);
}

EnumValue::EnumValue (const class Type* type) :
  Value (type),
  myValue (0)
{
  assert (getType ()->getKind () == Type::tEnum);
  bottom ();
}

EnumValue::EnumValue (const class EnumValue& old) :
  Value (old.getType ()),
  myValue (old.getValue ())
{
  assert (getType ()->getKind () == Type::tEnum);
}

EnumValue::~EnumValue ()
{
}

void
EnumValue::bottom ()
{
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getFirstValue ();
    assert (!v || v->getKind () == getKind ());
    myValue = v
      ? static_cast<const class EnumValue*>(v)->myValue
      : static_cast<const class EnumType*>(getType ())->getFirstEnum ();
  }
  else
    myValue = static_cast<const class EnumType*>(getType ())->getFirstEnum ();
}

void
EnumValue::top ()
{
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getLastValue ();
    assert (!v || v->getKind () == getKind ());
    myValue = v
      ? static_cast<const class EnumValue*>(v)->myValue
      : static_cast<const class EnumType*>(getType ())->getLastEnum ();
  }
  else
    myValue = static_cast<const class EnumType*>(getType ())->getLastEnum ();
}

bool
EnumValue::increment ()
{
  const class EnumType& type = *static_cast<const class EnumType*>(getType ());
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getNextHigh (this);
    if (v) {
      assert (v->getKind () == getKind ());
      if (myValue == static_cast<const class EnumValue*>(v)->myValue) {
	v = getType ()->getConstraint ()->getNextLow (this);
	if (!v) {
	  bottom ();
	  return false;
	}
	assert (v->getKind () == getKind ());
	myValue = static_cast<const class EnumValue*>(v)->myValue;
	return true;
      }
      else {
	int_t value = myValue;

	if (type.hasValue (value) && type.getNextEnum (value))
	  myValue = value;
	else
	  myValue++;
	return true;
      }
    }
  }

  if (type.hasValue (myValue))
    return type.getNextEnum (myValue);
  else if (myValue == INT_T_MAX) {
    bottom ();
    return false;
  }
  else {
    myValue++;
    return true;
  }
}

bool
EnumValue::decrement ()
{
  const class EnumType& type = *static_cast<const class EnumType*>(getType ());
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getPrevLow (this);
    if (v) {
      assert (v->getKind () == getKind ());
      if (myValue == static_cast<const class EnumValue*>(v)->myValue) {
	v = getType ()->getConstraint ()->getPrevHigh (this);
	if (!v) {
	  top ();
	  return false;
	}
	assert (v->getKind () == getKind ());
	myValue = static_cast<const class EnumValue*>(v)->myValue;
	return true;
      }
      else {
	int_t value = myValue;

	if (type.hasValue (value) && type.getPrevEnum (value))
	  myValue = value;
	else
	  myValue--;
	return true;
      }
    }
  }

  if (type.hasValue (myValue))
    return type.getPrevEnum (myValue);
  else if (myValue == INT_T_MIN) {
    top ();
    return false;
  }
  else {
    myValue--;
    return true;
  }
}

#include "Printer.h"

void
EnumValue::display (const class Printer& printer) const
{
  if (const char* const comp =
      static_cast<const class EnumType*>(getType ())->getEnumName (myValue))
    printer.print (comp);
  else
    printer.print (myValue);
}

#ifdef DEBUG
# include "util.h"

void
EnumValue::prettyPrint (unsigned indent) const
{
  if (const char* comp =
      static_cast<const class EnumType*>(getType ())->getEnumName (myValue))
    print (indent, "%s\n", comp);
  else
    print (indent, "%d\n", myValue);
}

#endif // DEBUG
