// Enumeration value class -*- c++ -*-

#ifndef ENUMVALUE_H_
# define ENUMVALUE_H_

# include "Value.h"
# include "typedefs.h"
# include <cassert>

/** Enumeration value */
class EnumValue : public Value {
public:
  // Constructors and destructors

  /** Constructs EnumValue
   * @param type	type of the value
   * @param value	the enumeration value
   */
  EnumValue (const class Type* type, int_t value);
  /** Constructs EnumValue, initializing it to the smallest possible value
   * @param type	type of the value
   */
  EnumValue (const class Type* type);
private:
  /** Copy constructor */
  EnumValue (const class EnumValue& old);
  /** Assignment operator */
  class EnumValue& operator= (const class EnumValue& old);
public:
  /** Destructs EnumValue */
  ~EnumValue ();
  /** Copies EnumValue */
  class Value* copy () const { return new class EnumValue (*this); }

  // Accessors

  /** Determine the kind of the value */
  enum Kind getKind () const { return vEnum; }

  /** Determine the value */
  int getValue () const { return myValue; }

  // Other methods

  /** Less-than comparison */
  bool operator< (const class EnumValue& other) const {
    assert (other.getType () == getType ());
    return myValue < other.myValue;
  }
  /** Less-or-equal comparison */
  bool operator<= (const class EnumValue& other) const {
    assert (other.getType () == getType ());
    return myValue <= other.myValue;
  }
  /** Equality comparison */
  bool operator== (const class EnumValue& other) const {
    assert (other.getType () == getType ());
    return myValue == other.myValue;
  }

  /** Difference operator
   * @param other	value to be substracted from this (may not be less)
   * @return		the difference
   */
  card_t operator- (const class EnumValue& other) const {
    return myValue - other.myValue;
  }

  /** Reset the value to the first value */
  void bottom ();
  /** Reset the value to the last value */
  void top ();
  /** Get the next value
   * @return	false if the value wrapped around; otherwise true
   */
  bool increment ();
  /** Get the previous value
   * @return	false if the value wrapped around; otherwise true
   */
  bool decrement ();

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

# ifdef DEBUG
  /**
   * Print this object
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The value */
  int_t myValue;
};

#endif // ENUMVALUE_H_
