// Identifier value class -*- c++ -*-

#ifndef IDVALUE_H_
# define IDVALUE_H_

# include "Value.h"

/** Identifier value */
class IdValue : public Value {
public:
  // Constructors and destructors

  /** Constructs IdValue
   * @param type	type of the value
   * @param value	the identifier value
   */
  IdValue (const class Type* type, card_t value);
private:
  /** Copy constructor */
  IdValue (const class IdValue& old);
  /** Assignment operator */
  class IdValue& operator= (const class IdValue& old);
public:
  /** Destructs IdValue */
  ~IdValue ();
  /** Copies IdValue */
  class Value* copy () const { return new class IdValue (*this); }

  // Accessors

  /** Determine the kind of the value */
  enum Kind getKind () const { return vId; }

  /** Determine the value */
  card_t getValue () const { return myValue; }

  // Other methods

  /** Less-than comparison */
  bool operator< (const class IdValue& other) const {
    assert (other.getType () == getType ());
    return myValue < other.myValue;
  }
  /** Less-or-equal comparison */
  bool operator<= (const class IdValue& other) const {
    assert (other.getType () == getType ());
    return myValue <= other.myValue;
  }
  /** Equality comparison */
  bool operator== (const class IdValue& other) const {
    assert (other.getType () == getType ());
    return myValue == other.myValue;
  }

  /** Difference operator
   * @param other	value to be substracted from this (may not be less)
   * @return		the difference
   */
  card_t operator- (const class IdValue& other) const {
    return myValue - other.myValue;
  }

  /** Reset the value to the first value */
  void bottom ();
  /** Reset the value to the last value */
  void top ();
  /** Get the next value
   * @return	false if the value wrapped around; otherwise true
   */
  bool increment ();
  /** Get the previous value
   * @return	false if the value wrapped around; otherwise true
   */
  bool decrement ();

  /** Convert the value to another type if possible
   * @param type	type to be casted to
   * @return		the converted value, or NULL
   */
  class Value* cast (const class Type& type);

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

# ifdef DEBUG
  /**
   * Print this object
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The value */
  card_t myValue;
};

#endif // IDVALUE_H_
