// Integer value class -*- c++ -*-

#include "IntValue.h"
#include "Type.h"
#include "Constraint.h"

IntValue::IntValue (const class Type* type, int_t value) :
  Value (type),
  myValue (value)
{
  assert (getType ()->getKind () == Type::tInt);
}

IntValue::IntValue (const class Type* type) :
  Value (type),
  myValue (0)
{
  assert (getType ()->getKind () == Type::tInt);
  bottom ();
}

IntValue::IntValue (const class IntValue& old) :
  Value (old.getType ()),
  myValue (old.getValue ())
{
  assert (getType ()->getKind () == Type::tInt);
}

IntValue::~IntValue ()
{
}

void
IntValue::bottom ()
{
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getFirstValue ();
    assert (!v || v->getKind () == getKind ());
    myValue = v ? static_cast<const class IntValue*>(v)->myValue : INT_T_MIN;
  }
  else
    myValue = INT_T_MIN;
}

void
IntValue::top ()
{
  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getLastValue ();
    assert (!v || v->getKind () == getKind ());
    myValue = v ? static_cast<const class IntValue*>(v)->myValue : INT_T_MAX;
  }
  else
    myValue = INT_T_MAX;
}

bool
IntValue::increment ()
{
  if (myValue == INT_T_MAX) {
    bottom ();
    return false;
  }

  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getNextHigh (this);
    if (v) {
      assert (v->getKind () == getKind ());
      if (myValue == static_cast<const class IntValue*>(v)->myValue) {
	v = getType ()->getConstraint ()->getNextLow (this);
	if (!v) {
	  bottom ();
	  return false;
	}
	assert (v->getKind () == getKind ());
	myValue = static_cast<const class IntValue*>(v)->myValue;
	return true;
      }
    }
  }

  myValue++;
  return true;
}

bool
IntValue::decrement ()
{
  if (myValue == INT_T_MAX) {
    top ();
    return false;
  }

  if (getType ()->getConstraint ()) {
    const class Value* v = getType ()->getConstraint ()->getPrevLow (this);
    if (v) {
      assert (v->getKind () == getKind ());
      if (myValue == static_cast<const class IntValue*>(v)->myValue) {
	v = getType ()->getConstraint ()->getPrevHigh (this);
	if (!v) {
	  top ();
	  return false;
	}
	assert (v->getKind () == getKind ());
	myValue = static_cast<const class IntValue*>(v)->myValue;
	return true;
      }
    }
  }

  myValue--;
  return true;
}

#include "Printer.h"

void
IntValue::display (const class Printer& printer) const
{
  printer.print (myValue);
}

#ifdef DEBUG
# include "util.h"

void
IntValue::prettyPrint (unsigned indent) const
{
  print (indent, "%d\n", myValue);
}

#endif // DEBUG
