// Integer value class -*- c++ -*-

#ifndef INTVALUE_H_
# define INTVALUE_H_

# include "Value.h"
# include "typedefs.h"

/** Integer value */
class IntValue : public Value {
public:
  // Constructors and destructors

  /** Constructs IntValue
   * @param type	type of the value
   * @param value	the integer value
   */
  IntValue (const class Type* type, int_t value);
  /** Constructs IntValue, initializing it to the smallest possible value
   * @param type	type of the value
   */
  IntValue (const class Type* type);
private:
  /** Copy constructor */
  IntValue (const class IntValue& old);
  /** Assignment operator */
  class IntValue& operator= (const class IntValue& old);
public:
  /** Destructs IntValue */
  ~IntValue ();
  /** Copies IntValue */
  class Value* copy () const { return new class IntValue (*this); }

  // Accessors

  /** Determine the kind of the value */
  enum Kind getKind () const { return vInt; }

  /** Determine the value */
  int_t getValue () const { return myValue; }

  /** Set the value */
  void setValue (int_t value) { myValue = value; }

  // Other methods

  /** Less-than comparison */
  bool operator< (const class IntValue& other) const {
    return myValue < other.myValue;
  }
  /** Less-or-equal comparison */
  bool operator<= (const class IntValue& other) const {
    return myValue <= other.myValue;
  }
  /** Equality comparison */
  bool operator== (const class IntValue& other) const {
    return myValue == other.myValue;
  }

  /** Difference operator
   * @param other	value to be substracted from this (may not be less)
   * @return		the difference
   */
  card_t operator- (const class IntValue& other) const {
    return myValue - other.myValue;
  }

  /** Reset the value to the first value */
  void bottom ();
  /** Reset the value to the last value */
  void top ();
  /** Get the next value
   * @return	false if the value wrapped around; otherwise true
   */
  bool increment ();
  /** Get the previous value
   * @return	false if the value wrapped around; otherwise true
   */
  bool decrement ();

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

# ifdef DEBUG
  /**
   * Print this object
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The value */
  int_t myValue;
};

#endif // INTVALUE_H_
