// Structure value class -*- c++ -*-

#ifndef STRUCTVALUE_H_
# define STRUCTVALUE_H_

# include "Value.h"
# include "ValueList.h"

/** Structure value */
class StructValue : public Value {
public:
  // Constructors and destructors

  /** Constructs StructValue
   * @param type	type of the value
   */
  StructValue (const class Type* type);
private:
  /** Copy constructor */
  StructValue (const class StructValue& old);
  /** Assignment operator */
  class StructValue& operator= (const class StructValue& old);
public:
  /** Destructs StructValue */
  ~StructValue ();
  /** Copies StructValue */
  class Value* copy () const { return new class StructValue (*this); }

  // Accessors

  /** Determine the kind of the value */
  enum Kind getKind () const { return vStruct; }

  /** Determine the number of components in the struct */
  card_t getSize () const { return myComponents.getSize (); }
  /** Address a component by index */
  const class Value* operator[] (card_t index) const {
    return myComponents[index];
  }
  /** Address a component by index */
  class Value*& operator[] (card_t index) {
    return myComponents[index];
  }

  // Other methods

  /** Less-than comparison */
  bool operator< (const class StructValue& other) const;
  /** Less-or-equal comparison */
  bool operator<= (const class StructValue& other) const;
  /** Equality comparison */
  bool operator== (const class StructValue& other) const;

  /** Difference operator
   * @param other	value to be substracted from this (may not be less)
   * @return		the difference
   */
  card_t operator- (const class StructValue& other) const;

  /** Reset the value to the first value */
  void bottom ();
  /** Reset the value to the last value */
  void top ();
  /** Get the next value
   * @return	false if the value wrapped around; otherwise true
   */
  bool increment ();
  /** Get the previous value
   * @return	false if the value wrapped around; otherwise true
   */
  bool decrement ();

  /** Convert the value to another type if possible
   * @param type	type to be casted to
   * @return		the converted value, or NULL
   */
  class Value* cast (const class Type& type);

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

# ifdef DEBUG
  /**
   * Print this object
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** The components */
  class ValueList myComponents;
};

#endif // STRUCTVALUE_H_
