// Union value class -*- c++ -*-

#ifndef UNIONVALUE_H_
# define UNIONVALUE_H_

# include "Value.h"
# include "typedefs.h"

/** Union value */
class UnionValue : public Value {
public:
  // Constructors and destructors

  /** Constructs UnionValue
   * @param type	the union type
   * @param index	index of the selected component
   * @param value	value of the selected component
   */
  UnionValue (const class Type* type, card_t index, class Value* value);
private:
  /** Copy constructor */
  UnionValue (const class UnionValue& old);
  /** Assignment operator */
  class UnionValue& operator= (const class UnionValue& old);
public:
  /** Destructs UnionValue */
  ~UnionValue ();
  /** Copies UnionValue */
  class Value* copy () const { return new class UnionValue (*this); }

  // Accessors

  /** Determine the kind of the value */
  enum Kind getKind () const { return vUnion; }

  /** Determine the index of the present component */
  card_t getIndex () const { return myIndex; }
  /** Determine the value of the present component */
  const class Value* getValue () const { return myValue; }

  // Other methods

  /** Less-than comparison */
  bool operator< (const class UnionValue& other) const;
  /** Less-or-equal comparison */
  bool operator<= (const class UnionValue& other) const;
  /** Equality comparison */
  bool operator== (const class UnionValue& other) const;

  /** Difference operator
   * @param other	value to be substracted from this (may not be less)
   * @return		the difference
   */
  card_t operator- (const class UnionValue& other) const;

  /** Reset the value to the first value */
  void bottom ();
  /** Reset the value to the last value */
  void top ();
  /** Get the next value
   * @return	false if the value wrapped around; otherwise true
   */
  bool increment ();
  /** Get the previous value
   * @return	false if the value wrapped around; otherwise true
   */
  bool decrement ();

  /** Convert the value to another type if possible
   * @param type	type to be casted to
   * @return		the converted value, or NULL
   */
  class Value* cast (const class Type& type);

  /** Display this object
   * @param printer	The printer object
   */
  void display (const class Printer& printer) const;

# ifdef DEBUG
  /**
   * Print this object
   * @param indent	Level of indentation
   */
  void prettyPrint (unsigned indent) const;
# endif // DEBUG

private:
  /** Index of the referenced union component */
  card_t myIndex;
  /** Value of the component */
  class Value* myValue;
};

#endif // UNIONVALUE_H_
