// Value base class -*- c++ -*-

#include "allValues.h"
#include "UnionType.h"

Value::Value (const class Type* type) :
  myType (type)
{
  assert (!!getType ());
}

Value::~Value ()
{
}

bool
Value::operator< (const class Value& other) const
{
  assert (other.getKind () == getKind ());
  switch (getKind ()) {
  case vInt:
    return static_cast<const class IntValue&>(*this) <
      static_cast<const class IntValue&>(other);
  case vBool:
    return static_cast<const class BoolValue&>(*this) <
      static_cast<const class BoolValue&>(other);
  case vChar:
    return static_cast<const class CharValue&>(*this) <
      static_cast<const class CharValue&>(other);
  case vEnum:
    return static_cast<const class EnumValue&>(*this) <
      static_cast<const class EnumValue&>(other);
  case vId:
    return static_cast<const class IdValue&>(*this) <
      static_cast<const class IdValue&>(other);
  case vStruct:
    return static_cast<const class StructValue&>(*this) <
      static_cast<const class StructValue&>(other);
  case vUnion:
    return static_cast<const class UnionValue&>(*this) <
      static_cast<const class UnionValue&>(other);
  case vVector:
    return static_cast<const class VectorValue&>(*this) <
      static_cast<const class VectorValue&>(other);
  case vBuffer:
    return static_cast<const class BufferValue&>(*this) <
      static_cast<const class BufferValue&>(other);
  }
  assert (false);
  return false;
}

bool
Value::operator<= (const class Value& other) const
{
  assert (other.getKind () == getKind ());
  switch (getKind ()) {
  case vInt:
    return static_cast<const class IntValue&>(*this) <=
      static_cast<const class IntValue&>(other);
  case vBool:
    return static_cast<const class BoolValue&>(*this) <=
      static_cast<const class BoolValue&>(other);
  case vChar:
    return static_cast<const class CharValue&>(*this) <=
      static_cast<const class CharValue&>(other);
  case vEnum:
    return static_cast<const class EnumValue&>(*this) <=
      static_cast<const class EnumValue&>(other);
  case vId:
    return static_cast<const class IdValue&>(*this) <=
      static_cast<const class IdValue&>(other);
  case vStruct:
    return static_cast<const class StructValue&>(*this) <=
      static_cast<const class StructValue&>(other);
  case vUnion:
    return static_cast<const class UnionValue&>(*this) <=
      static_cast<const class UnionValue&>(other);
  case vVector:
    return static_cast<const class VectorValue&>(*this) <=
      static_cast<const class VectorValue&>(other);
  case vBuffer:
    return static_cast<const class BufferValue&>(*this) <=
      static_cast<const class BufferValue&>(other);
  }
  assert (false);
  return false;
}

bool
Value::operator== (const class Value& other) const
{
  assert (other.getKind () == getKind ());
  switch (getKind ()) {
  case vInt:
    return static_cast<const class IntValue&>(*this) ==
      static_cast<const class IntValue&>(other);
  case vBool:
    return static_cast<const class BoolValue&>(*this) ==
      static_cast<const class BoolValue&>(other);
  case vChar:
    return static_cast<const class CharValue&>(*this) ==
      static_cast<const class CharValue&>(other);
  case vEnum:
    return static_cast<const class EnumValue&>(*this) ==
      static_cast<const class EnumValue&>(other);
  case vId:
    return static_cast<const class IdValue&>(*this) ==
      static_cast<const class IdValue&>(other);
  case vStruct:
    return static_cast<const class StructValue&>(*this) ==
      static_cast<const class StructValue&>(other);
  case vUnion:
    return static_cast<const class UnionValue&>(*this) ==
      static_cast<const class UnionValue&>(other);
  case vVector:
    return static_cast<const class VectorValue&>(*this) ==
      static_cast<const class VectorValue&>(other);
  case vBuffer:
    return static_cast<const class BufferValue&>(*this) ==
      static_cast<const class BufferValue&>(other);
  }
  assert (false);
  return false;
}

card_t
Value::operator- (const class Value& other) const
{
  assert (other.getKind () == getKind ());
  assert (other <= *this);
  switch (getKind ()) {
  case vInt:
    return static_cast<const class IntValue&>(*this) -
      static_cast<const class IntValue&>(other);
  case vBool:
    return static_cast<const class BoolValue&>(*this) -
      static_cast<const class BoolValue&>(other);
  case vChar:
    return static_cast<const class CharValue&>(*this) -
      static_cast<const class CharValue&>(other);
  case vEnum:
    return static_cast<const class EnumValue&>(*this) -
      static_cast<const class EnumValue&>(other);
  case vId:
    return static_cast<const class IdValue&>(*this) -
      static_cast<const class IdValue&>(other);
  case vStruct:
    return static_cast<const class StructValue&>(*this) -
      static_cast<const class StructValue&>(other);
  case vUnion:
    return static_cast<const class UnionValue&>(*this) -
      static_cast<const class UnionValue&>(other);
  case vVector:
    return static_cast<const class VectorValue&>(*this) -
      static_cast<const class VectorValue&>(other);
  case vBuffer:
    return static_cast<const class BufferValue&>(*this) -
      static_cast<const class BufferValue&>(other);
  }
  assert (false);
  return 0;
}

class Value*
Value::cast (const class Type& type)
{
  if (type.getKind () == Type::tUnion)
    return static_cast<const class UnionType&>(type).cast (*this);
  else if (type.isConstrained (this)) {
    myType = &type;
    return this;
  }
  else {
    delete this;
    return 0;
  }
}
