// Extendible reusable string buffer -*- c++ -*-

#ifndef _STRINGBUFFER_H_
# define _STRINGBUFFER_H_

/** Extendible reusable string buffer */

class StringBuffer {
public:
  //  Constructors and destructors

  /** Constructs a StringBuffer */
  StringBuffer ();
  /** Copy constructor */
  StringBuffer (const class StringBuffer& old);
  /** Assignment operator */
  class StringBuffer& operator= (const class StringBuffer& old);
  /** Destructs StringBuffer */
  ~StringBuffer ();

  // Accessors

  /** Get current string length */
  unsigned getLength () const { return myLength; }
  /** Get maximum string length */
  unsigned getMaxLength () const { return myMaxLength; }
  /** Get the string buffer */
  const char* getString () const { return myString; }
  /** Assignment operator
   * @param string	NUL-terminated string to be assigned
   */
  class StringBuffer& operator= (const char* string);

  /**
   * Guarantee that getMaxLength () <= length and clear the string.
   * @param length	length of string requested
   * @returns		pointer to string buffer (relocated when necessary)
   */
  char* create (unsigned length);

  /**
   * Guarantee that getMaxLength () - getLength () <= length
   * and set myLength += length.
   * @param length	amount of additional space requested
   * @returns		pointer to string buffer (relocated when necessary)
   */
  char* extend (unsigned length);

private:
  /** The string buffer */
  char* myString;
  /** Current string length, excluding the terminating NULL character */
  unsigned myLength;
  /** Maximum string length, excluding the terminating NULL character */
  unsigned myMaxLength;
};

#endif // _STRINGBUFFER_H_
