// Variable stack map to be used with nested scoping -*- c++ -*-

#include "VariableStackMap.h"
#include "VariableDefinition.h"
#include <cassert>

VariableStackMap::~VariableStackMap ()
{
  for (Stack::iterator i = myStack.begin (); i != myStack.end (); i++)
    delete *i;
}

void
VariableStackMap::push (class VariableDefinition* v)
{
  myStack.push_front (v);
  Map::iterator i = myMap.find (v->getName ());
  if (i != myMap.end ())
    (*i).second.push_front (v);
  else
    myMap[v->getName ()].push_front (v);
}

void
VariableStackMap::pop ()
{
  assert (!myStack.empty ());
  Map::iterator i = myMap.find ((*myStack.begin ())->getName ());
  assert (i != myMap.end ());
  Stack& s = i->second;
  s.erase (s.begin ());
  myStack.erase (myStack.begin ());
  if (s.empty ())
    myMap.erase (i);
}

void
VariableStackMap::clear ()
{
  for (Stack::const_iterator i = myStack.begin (); i != myStack.end (); i++)
    delete *i;
  myMap.clear ();
  myStack.clear ();
}

class VariableDefinition*
VariableStackMap::find (const char* name) const
{
  Map::const_iterator i = myMap.find (name);
  if (i == myMap.end ())
    return NULL;

  const Stack& s = (*i).second;
  if (s.empty ())
    return NULL;

  return *s.begin ();
}
