// Variable stack map to be used with nested scoping -*- c++ -*-

#ifndef VARIABLESTACKMAP_H_
# define VARIABLESTACKMAP_H_

# include <cstring>
# include <map>
# include <slist>

/** Variable stack map to be used with nested scoping */
class VariableStackMap {
  /** String comparator */
  struct ltstr {
    bool operator () (const char* s1, const char* s2) const {
      return strcmp (s1, s2) < 0;
    }
  };
  // Typedefs

  typedef std::slist<class VariableDefinition*> Stack;
  typedef std::map<const char*,Stack,struct ltstr> Map;

  // Constructors and destructors
public:
  /** Construct VariableStackMap */
  VariableStackMap () : myMap (), myStack () {}
private:
  /** Copy constructor */
  VariableStackMap (const class VariableStackMap& old);
  /** Assignment operator */
  class VariableStackMap& operator= (const class VariableStackMap& old);
public:
  /** Destructor */
  ~VariableStackMap ();

  // Accessors

  /** Push a variable definition on the stack
   * @param v		variable definition to be pushed on the stack
   */
  void push (class VariableDefinition* v);
  /** Get the topmost quantifier from the stack */
  class VariableDefinition* top () const { return *myStack.begin (); }
  /** Pop a quantifier from the stack */
  void pop ();
  /** Determine whether the container is empty */
  bool empty () const { return myStack.empty (); }
  /** Clear the container */
  void clear ();
  /** Find the topmost variable carrying the specified variable name
   * @param name	name being sought
   */
  class VariableDefinition* find (const char* name) const;

  // Members
private:
  Map myMap;
  Stack myStack;
};

#endif // VARIABLESTACKMAP_H_
