/* -*- c++ -*- Scanner for Maria's Petri Net language */

%option nostdinit
%option stack
%option prefix="pn"
%option 8bit
%option caseful
%option warn
%option pointer
%option never-interactive
%option nodefault
%option noinput
%option nounput
%option noyywrap
%option noyy_top_state
%option noyy_scan_buffer
%option noyy_scan_bytes
%option noyy_scan_string

%{
# include <cstdio>
# include <climits>
# include <cstring>
# include <cstdarg>

# include "typedefs.h"
# include "maria.tab.h"

# include "StringBuffer.h"
# include "util.h"

# include <list>
# include <stack>
  /* Stack of include files */
  struct input {
    /** name of the input file */
    class StringBuffer name;
    /** input line number */
    unsigned lineno;
    /** handle to the input file */
    FILE* file;
    /** buffer state */
    YY_BUFFER_STATE bs;
  };
  static std::stack<struct input, std::list<struct input> > in;

  /* Dynamically growing string buffer */
  static class StringBuffer sbuf;
  /* String handling macros.  All of these use sbuf. */
# define CREAT memcpy (sbuf.create (yyleng), yytext, yyleng + 1)
# define INITSTR sbuf.create (0)
# define EXTEND memcpy (sbuf.extend (yyleng) + sbuf.getLength () - yyleng, \
			yytext, yyleng + 1)
# define APPEND(c) (&((sbuf.extend (1))[sbuf.getLength () - 1] = c))[1] = 0
# define COPY pnlval.s = newString (sbuf.getString ())
# define COPYBUF pnlval.sbuf = new class StringBuffer (sbuf)

  /* Start condition stack handling */
# define SC_FLUSH	yy_start_stack_ptr = 0, BEGIN (INITIAL)
# define SC_POP		yy_pop_state ()
# define SC_POP_B	do { SC_POP; yyless (0); } while (0)
# define SC_PUSH(s)	yy_push_state (s)
# define SC_SWITCH(s)	BEGIN (s)
# define SC_SWITCH_B(s)	do { yyless (0); SC_SWITCH (s); } while (0)
# define SC_CALL(s,r)	do { rt = r; SC_PUSH (ret); SC_PUSH (s); } while (0)
# define SC_USE(s,r)	do { SC_SWITCH (r); SC_PUSH (s); } while (0)

  /** initial source file name */
  extern const char* netname;
  /** Name of Petri Nets parsed from the command line */
  extern const char* commandline;
  /** Petri Net model to be parsed from the command line */
  extern const char* netstring;
  /** Cursor to netstring */
  static const char* netptr = NULL;

  /** source file line number */
  static unsigned lineno = 1;
  /** source file name */
  static class StringBuffer filename;

  /** Token to be returned to the parser */
  static int rt;
  /** Flag: input constant overflow */
  static bool overflow = false;
  /** Number of digits read in an octal or hexadecimal character constant */
  static unsigned digits = 0;
  /** Character constant being parsed in a string */
  static char character;

  /** warning flag (true=print warnings) */
  bool pnwarnings = true;
  /** number of errors encountered */
  unsigned pnerrors = 0;

  /** The place that is currently being processed */
  extern class Place* currentPlace;
# include "Place.h"
  /** The transition that is currently being processed */
  extern class Transition* currentTransition;
# include "Transition.h"

  /**
   * generic compiler warning reporting function
   * @param s		warning message
   */
  void pnwarn (const char* s);

  /**
   * generic printf-like compiler warning reporting function
   * @param fmt		printf format string
   * @param ...		variable arguments
   */
  void pnwarnf (const char* fmt, ...)
# ifdef __GNUC__
    __attribute__ ((format (printf, 1, 2)))
# endif /* __GNUC__ */
    ;

  /**
   * generic parser error reporting function
   * @param s		error message
   */
  void pnerror (const char* s);

  /**
   * generic printf-like parser error reporting function
   * @param fmt		printf format string
   * @param ...		variable arguments
   */
  void pnerrorf (const char* fmt, ...)
# ifdef __GNUC__
    __attribute__ ((format (printf, 1, 2)))
# endif /* __GNUC__ */
    ;

  /** Open an input file.
   * @param name	name of the input file
   * @return		a descriptor to the file, or NULL on error
   */
  extern FILE* openfile (const char* name);
  /** Switch files on EOF (return to the file that included the current file)
   * @return	true if there was a parent file
   */
  static bool popInclude ();

# define NEXTBUF do if (!popInclude ()) { SC_FLUSH; yyterminate (); } while (0)
# define YY_USER_INIT NEXTBUF

# define YY_INPUT(buf,result,max_size)					  \
  if (!pnin || pnin == stdin) {						  \
    if (!netptr)							  \
      result = 0;							  \
    else if (*netptr) {							  \
      if ((result = strlen (netptr)) > max_size)			  \
	result = max_size;						  \
      memcpy (buf, netptr, result);					  \
      netptr += result;							  \
    }									  \
    else if (max_size >= 1)						  \
      *buf = '\n', netptr = NULL, result = 1;				  \
    else								  \
      result = 0;							  \
  }									  \
  else if (!(result = fread (buf, 1, max_size, pnin)) && ferror (pnin)) { \
    pnerror ("cannot read file"); NEXTBUF;				  \
  }
%}

/* Start conditions */
/* returning to the parser */
%x ret
/* inside a comment */
%x comment

/* parsing a preprocessor directive */
%x pr
/* parsing a preprocessor directive, ignoring the rest of the line */
%x pr_ignore
/* parsing a preprocessor directive "line" */
%x pr_line
/* parsing a preprocessor directive "line", got line number */
%x pr_line_number
/* parsing a preprocessor directive "line", got file name */
%x pr_line_name
/* parsing a preprocessor directive "include" */
%x pr_incl
/* parsing a preprocessor directive "include", got file name */
%x pr_incl_name

/* parsing an octal constant */
%x octal
/* parsing a hexadecimal constant */
%x hexadecimal
/* parsing a decimal constant */
%x decimal

/* parsing a name */
%x pnname
/* parsing a character constant */
%x pnchar
/* parsing an octal character constant */
%x pnchar_oct
/* parsing a hexadecimal character constant */
%x pnchar_hex
/* parsing a string constant */
%x pnstring
/* parsing an octal character constant in a string */
%x pnstring_oct
/* parsing a hexadecimal character constant in a string */
%x pnstring_hex

ws		[ \t\r\v\f]+
nn		[^\\A-Za-z0-9_]

%%

<ret>.|\n	{ SC_POP_B; if (rt == STRING) COPYBUF; return rt; }
<ret><<EOF>>	{ if (rt == STRING) COPYBUF; SC_POP; NEXTBUF; return rt; }

<comment>{
  [^*\n]*
  [^*\n]*\n		lineno++;
  "*"+[^*/\n]*
  "*"+[^*/\n]*\n	lineno++;
  "*"+"/"		SC_POP;
  <<EOF>>		{ pnerror ("EOF in comment"); SC_POP; NEXTBUF; }
}

<pr,pr_line,pr_line_number,pr_line_name,pr_incl,pr_incl_name>{
  {ws}+
  "//"[^\n]*
  "/*"		SC_PUSH (comment);
  <<EOF>>	{ pnerror ("EOF in preprocessor directive"); SC_POP; NEXTBUF; }
}

<pr>{
  [A-Za-z_][A-Za-z0-9_]* {
    if (!strcmp (yytext, "line"))
      SC_SWITCH (pr_line);
    else if (!strcmp (yytext, "include"))
      SC_SWITCH (pr_incl);
    else {
      pnerrorf ("unknown preprocessor directive #%s", yytext);
      SC_SWITCH (pr_ignore);
    }
  }
  !			{
    SC_SWITCH (pr_ignore); /* #! interpreter command */
  }
  .			{
    pnerror ("error in preprocessor directive"); SC_SWITCH (pr_ignore);
  }
  \n			{
    pnerror ("expected a preprocessor directive"); SC_POP; lineno++;
  }
}

<pr_ignore>{
  [^\n]+		SC_POP;
  \n			{ SC_POP; lineno++; }
  <<EOF>>		{ SC_POP; NEXTBUF; }
}

<pr_line>{
  0x		{
    pnlval.i = 0, overflow = false;
    SC_USE (hexadecimal, pr_line_number);
  }
  0		{
    pnlval.i = 0, overflow = false;
    SC_USE (octal, pr_line_number);
  }
  [1-9]		{
    pnlval.i = *yytext - '0', overflow = false;
    SC_USE (decimal, pr_line_number);
  }
  \"			{ INITSTR; SC_USE (pnstring, pr_line_name); }
  \n			{ lineno++; SC_POP; }
  .			{
    pnerror ("error in preprocessor directive #line");
    SC_SWITCH (pr_ignore);
  }
}

<pr_line_number>.|\n	{
  SC_SWITCH_B (pr_line);
  lineno = pnlval.i - 1;
}

<pr_line_name>.|\n	{ SC_SWITCH_B (pr_line); filename = sbuf; }

<pr_incl>{
  \"			{ INITSTR; SC_USE (pnstring, pr_incl_name); }
  \n			{ lineno++; SC_POP; }
  .			{
    pnerror ("error in preprocessor directive #include");
    SC_SWITCH_B (pr_ignore);
  }
}

<pr_incl_name>.|\n	{
  SC_POP_B;
  if (FILE* f = openfile (sbuf.getString ())) {
    struct input is = { filename, lineno, pnin, YY_CURRENT_BUFFER };
    in.push (is);
    filename = sbuf;
    lineno = 1;
    pnin = f;
    yy_switch_to_buffer (yy_create_buffer (pnin, YY_BUF_SIZE));
  }
  else
    pnerrorf ("file `%s' not found", sbuf.getString ());
}

{ws}+
\n		lineno++;
"/*"		SC_PUSH (comment);
"//"[^\n]*
^#		SC_PUSH (pr);

"enum"/{nn}		return ENUM;
"id"/{nn}		return ID;
"union"/{nn}		return UNION;
"queue"/{nn}		return QUEUE;
"stack"/{nn}		return STACK;
"is"/{nn}		return IS;
"struct"/{nn}		return STRUCT;
"always"/{nn}		return ALWAYS;
"exists"/{nn}		return EXISTS;
"until"/{nn}		return UNTIL;
"release"/{nn}		return RELEASE;
"subset"/{nn}		return SUBSET;
"intersect"/{nn}	return INTERSECT;
"equals"/{nn}		return EQUALS;
"minus"/{nn}		return MINUS;
"atom"/{nn}		return ATOM;
"empty"/{nn}		return EMPTY;

"typedef"/{nn}		return TYPEDEF;
"place"/{nn}		return PLACE;
"trans"/{nn}		return TRANS;
"in"/{nn}		return IN;
"out"/{nn}		return OUT;
"gate"/{nn}		return GATE;
"tester"/{nn}		return TESTER;
"reject"/{nn}		return REJECT_;
"deadlock"/{nn}		return DEADLOCK;
"livelock"/{nn}		return LIVELOCK;
"infinite"/{nn}		return INFINITE;
"verify"/{nn}		return VERIFY;
"true"/{nn}		return TRUE_;
"false"/{nn}		return FALSE_;
"undefined"/{nn}	return UNDEFINED;
"fatal"/{nn}		return FATAL;

"cardinality"/{nn}	return CARDINALITY;

"=>"		return IMPL;
"^^"		return XOR;
"<=>"		return EQUIV;
"||"		return OR;
"&&"		return AND;
"<>"		return FINALLY;
"[]"		return GLOBALLY;
"()"		return NEXT;

"=="		return EQ;
"!="		return NE;
">="		return GE;
"<="		return LE;
"<<"		return ROL;
">>"		return ROR;
".."		return RANGE;

0x		{
  pnlval.i = 0, overflow = false; SC_CALL (hexadecimal, NUMBER);
}
0		{
  pnlval.i = 0, overflow = false; SC_CALL (octal, NUMBER);
}
[1-9]		{
  pnlval.i = *yytext - '0', overflow = false; SC_CALL (decimal, NUMBER);
}

<hexadecimal>{
  [0-9]		{
    if (overflow);
    else if (pnlval.i < 0 || pnlval.i > INT_MAX >> 3)
      pnerror ("hexadecimal constant too large"), overflow = true;
    else
      (pnlval.i <<= 4) |= *yytext - '0';
  }
  [A-F]		{
    if (overflow);
    else if (pnlval.i < 0 || pnlval.i > INT_MAX >> 3)
      pnerror ("hexadecimal constant too large"), overflow = true;
    else
      (pnlval.i <<= 4) |= *yytext - 'A' + 0x0A;
  }
  [a-f]		{
    if (overflow);
    else if (pnlval.i < 0 || pnlval.i > INT_MAX >> 3)
      pnerror ("hexadecimal constant too large"), overflow = true;
    else
      (pnlval.i <<= 4) |= *yytext - 'a' + 0x0A;
  }
}

<octal>[0-7]	{
  if (overflow);
  else if (pnlval.i < 0 || pnlval.i > INT_MAX >> 2)
    pnerror ("octal constant too large"), overflow = true;
  else
    (pnlval.i <<= 3) |= *yytext - '0';
}

<decimal>[0-9]	{
  if (overflow);
  else if (pnlval.i < 0) {
    int v = pnlval.i * 10 - (*yytext - '0');
    if (v / 10 != pnlval.i)
      pnerror ("decimal constant too small"), overflow = true;
    else
      pnlval.i = v;
  }
  else {
    int v = pnlval.i * 10 + (*yytext - '0');
    if (v / 10 != pnlval.i)
      pnerror ("decimal constant too large"), overflow = true;
    else
      pnlval.i = v;
  }
}

<hexadecimal,octal,decimal>{
  [0-9a-zA-Z_]	{
    pnerror ("improperly terminated numeric constant");
    yyless (0); SC_POP;
  }
  .|\n		{
    yyless (0); SC_POP;
  }
  <<EOF>>	{
    SC_POP; pnerror ("EOF in numeric constant"); NEXTBUF;
  }
}

[A-Za-z_][A-Za-z0-9_]*	{ CREAT; SC_PUSH (pnname); }
\\.			{ INITSTR; APPEND (yytext[1]); SC_PUSH (pnname); }

<pnname>{
  \\\n{ws}*		lineno++;
  \\.			APPEND (yytext[1]);
  [A-Za-z0-9_]+		EXTEND;
  .|\n			{ SC_POP_B; COPY; return NAME; }
}

"'"			{ overflow = false; SC_PUSH (pnchar); }

<pnchar>{
  \'			{
    if (!overflow) {
      pnerror ("null char constant");
      pnlval.c = 0;
    }
    SC_POP; return CHARACTER;
  }
  \\\n{ws}*		lineno++;
  \\/[0-7]		{
    if (!overflow) {
      pnlval.c = 0, digits = 0, overflow = true; SC_SWITCH (pnchar_oct);
    }
    else
      pnerror ("char constant too large, ignoring `\\'");
  }
  \\x/[0-9a-fA-F]	{
    if (!overflow) {
      pnlval.c = 0, digits = 0, overflow = true; SC_SWITCH (pnchar_hex);
    }
    else
      pnerror ("char constant too large, ignoring `\\x'");
  }
  \\.			{
    if (!overflow) {
      overflow = true;
      switch (yytext[1]) {
      case 'a': pnlval.c = '\a'; break;
      case 'b': pnlval.c = '\b'; break;
      case 't': pnlval.c = '\t'; break;
      case 'n': pnlval.c = '\n'; break;
      case 'v': pnlval.c = '\v'; break;
      case 'f': pnlval.c = '\f'; break;
      case 'r': pnlval.c = '\r'; break;
      default:
	pnlval.c = yytext[1];
      }
    }
    else
      pnerrorf ("char constant too large, ignoring `\\%c'", yytext[1]);
  }
  .			{
    if (!overflow) {
      overflow = true;
      pnlval.c = *yytext;
    }
    else
      pnerrorf ("char constant too large, ignoring `%c'", *yytext);
  }
  \n			{
    lineno++;
    pnerror ("unterminated char constant");
    if (!overflow)
      pnlval.c = 0;
    SC_POP; return CHARACTER;
  }
}

<pnchar_oct>[0-7]	{
  (pnlval.c <<= 3) |= *yytext - '0';
  if (++digits >= 3 || (pnlval.c >> (CHAR_T_BIT - 3)))
    SC_SWITCH (pnchar);
}

<pnchar_hex>{
  [0-9] {
    (pnlval.c <<= 4) |= *yytext - '0';
    if (++digits >= 2)
      SC_SWITCH (pnchar);
  }
  [a-f] {
    (pnlval.c <<= 4) |= *yytext - 'a' + 0x0a;
    if (++digits >= 2)
      SC_SWITCH (pnchar);
  }
  [A-F] {
    (pnlval.c <<= 4) |= *yytext - 'A' + 0x0a;
    if (++digits >= 2)
      SC_SWITCH (pnchar);
  }
}

<pnchar_oct,pnchar_hex>{
  .|\n			SC_SWITCH_B (pnchar);
}

<pnchar,pnchar_oct,pnchar_hex><<EOF>>		{
  pnerror ("EOF in char constant"); SC_POP; NEXTBUF;
}

\"			{ INITSTR; SC_CALL (pnstring, STRING); }

<pnstring>{
  \"			SC_POP;
  \\\n{ws}*		lineno++;
  \\/[0-7]		{
    character = 0, digits = 0; SC_SWITCH (pnstring_oct);
  }
  \\x/[0-9a-fA-F]	{
    character = 0, digits = 0; SC_SWITCH (pnstring_hex);
  }
  \\x
  \\.			{
    switch (yytext[1]) {
    case 'a': APPEND ('\a'); break;
    case 'b': APPEND ('\b'); break;
    case 't': APPEND ('\t'); break;
    case 'n': APPEND ('\n'); break;
    case 'v': APPEND ('\v'); break;
    case 'f': APPEND ('\f'); break;
    case 'r': APPEND ('\r'); break;
    default:
      APPEND (yytext[1]);
    }
  }
  \n			{ EXTEND; lineno++; }
  [^\\\"\n]+		EXTEND;
  \\			{
    pnerror ("string constant terminated by `\\'"); EXTEND; SC_POP;
  }
  <<EOF>>		{
    pnerror ("EOF in string constant"); SC_POP; NEXTBUF;
  }
}

<pnstring_oct>[0-7]	{
  (character <<= 3) |= *yytext - '0';
  if (++digits >= 3 || (character >> (CHAR_BIT - 3))) {
    APPEND (character);
    SC_SWITCH (pnstring);
  }
}

<pnstring_hex>{
  [0-9] {
    (character <<= 4) |= *yytext - '0';
    if (++digits >= 2) {
      APPEND (character);
      SC_SWITCH (pnstring);
    }
  }
  [a-f] {
    (character <<= 4) |= *yytext - 'a' + 0x0a;
    if (++digits >= 2) {
      APPEND (character);
      SC_SWITCH (pnstring);
    }
  }
  [A-F] {
    (character <<= 4) |= *yytext - 'A' + 0x0a;
    if (++digits >= 2) {
      APPEND (character);
      SC_SWITCH (pnstring);
    }
  }
}

<pnstring_oct,pnstring_hex>{
  .|\n			{ APPEND (character); SC_SWITCH_B (pnstring); }
  <<EOF>>		{
    APPEND (character); SC_POP;
    pnerror ("EOF at escaped character in string constant"); NEXTBUF;
  }
}

[!#%&()*+,\-./:;<=>?\[\]^{|}~]	return *yytext;
\\\n			lineno++;
.			{
  pnerrorf ("unknown character 0x%02x",
	    static_cast<unsigned char>(*yytext));
}

<INITIAL><<EOF>>	NEXTBUF;

%%

void
pnwarn (const char* s)
{
  if (pnwarnings) {
    fprintf (stderr, "%s:%u:Warning:", filename.getString (), lineno);
    if (currentTransition)
      fprintf (stderr, "transition `%s':", currentTransition->getName ());
    else if (currentPlace)
      fprintf (stderr, "place `%s':", currentPlace->getName ());
    fprintf (stderr, "%s\n", s);
  }
}

void
pnwarnf (const char* fmt, ...)
{
  if (pnwarnings) {
    va_list ap;
    va_start (ap, fmt);
    fprintf (stderr, "%s:%u:Warning:", filename.getString (), lineno);
    if (currentTransition)
      fprintf (stderr, "transition `%s':", currentTransition->getName ());
    else if (currentPlace)
      fprintf (stderr, "place `%s':", currentPlace->getName ());
    vfprintf (stderr, fmt, ap);
    putc ('\n', stderr);
    va_end (ap);
  }
}

void
pnerror (const char* s)
{
  fprintf (stderr, "%s:%u:", filename.getString (), lineno);
  if (currentTransition)
    fprintf (stderr, "transition `%s':", currentTransition->getName ());
  else if (currentPlace)
    fprintf (stderr, "place `%s':", currentPlace->getName ());
  fprintf (stderr, "%s\n", s);
  pnerrors++;
}

void
pnerrorf (const char* fmt, ...)
{
  va_list ap;
  va_start (ap, fmt);
  fprintf (stderr, "%s:%u:", filename.getString (), lineno);
  if (currentTransition)
    fprintf (stderr, "transition `%s':", currentTransition->getName ());
  else if (currentPlace)
    fprintf (stderr, "place `%s':", currentPlace->getName ());
  vfprintf (stderr, fmt, ap);
  putc ('\n', stderr);
  va_end (ap);
  pnerrors++;
}

static bool
popInclude ()
{
  if (in.empty ()) {
    static const char* lastnetname;
    static const char* lastnetstring;
    if (lastnetname == netname && lastnetstring == netstring) {
      filename = netname;
      return false;
    }
    else if (filename = lastnetname = netname, netname == commandline) {
      pnin = NULL;
      if (lastnetstring == netstring) {
	yy_init = 1;
	return false;
      }
      netptr = lastnetstring = netstring;
      return true;
    }
    else if (!(pnin = openfile (lastnetname))) {
      perror (lastnetname);
      yy_init = 1;
      return false;
    }
    else {
      lineno = 1;
      return true;
    }
  }
  else {
    if (pnin && pnin != stdin) fclose (pnin);
    filename = in.top ().name;
    lineno = in.top ().lineno;
    pnin = in.top ().file;
    yy_delete_buffer (YY_CURRENT_BUFFER);
    yy_switch_to_buffer (in.top ().bs);
    in.pop ();
    return true;
  }
}
